## Colonize the Moon Mod Project
## Copyright Shin 2005
## (if any portion of this copyright should be found to be void, the remainder shall remain in full effect)
## 
## Artillery

from CvPythonExtensions import *
from types import *
import random
import cPickle
import UnitUtils
import Config

# globals
gc = CyGlobalContext()


def isArtillery(pUnit):
    artilleryDict = Config.getArtilleryDict()
    return artilleryDict.has_key(gc.getUnitInfo(pUnit.getUnitType()).getType())
    
def isBombard(pBombard):
    if(isinstance(pBombard, NoneType)):
        return False
        
    return (gc.getUnitInfo(pBombard.getUnitType()).getType() in Config.bombards)

def hasRoom(pUnit):
    artilleryDict = Config.getArtilleryDict()
    pArti = artilleryDict[gc.getUnitInfo(pUnit.getUnitType()).getType()] 
    return (pUnit.cargoSpaceAvailable(gc.getInfoTypeForString(pArti.getSpecialBombardUnitType()), gc.getInfoTypeForString("DOMAIN_AIR")) > 0)


# Create a bombard for the artillery piece 
def createBombard(pUnit):
    artilleryDict = Config.getArtilleryDict()           
    pArti = artilleryDict[gc.getUnitInfo(pUnit.getUnitType()).getType()]
    pArti.createBombard(pUnit)


def testPropriety(pBombard):
    artilleryDict = Config.getArtilleryDict()
    pUnit = pBombard.getTransportUnit()
    if(pUnit.isNone()):
        pBombard.setDamage(100, True)
        return False
    
    if(isArtillery(pUnit)):
        pArti = artilleryDict[gc.getUnitInfo(pUnit.getUnitType()).getType()]
        if(pArti.isValidBombard(pBombard)):
            if(pBombard.movesLeft() == 0):
                pBombard.kill(False, 0)
                return False
            return True
        else:
            pBombard.setDamage(100, True)
            return False
    else:
        pBombard.setDamage(100, True)
        return False


def checkForCapacityPromotion(pUnit, iPromotion):
    artilleryDict = Config.getArtilleryDict()
    pArti = artilleryDict[gc.getUnitInfo(pUnit.getUnitType()).getType()]
    newUnit = pArti.checkForCapacityPromotion(pUnit, iPromotion)
    return newUnit


class Artillery:
    def __init__(self, baseType, bombardType, specBombardType, capType, capPromo):
        self.CapacityPromotedType = capType
        self.CapacityPromotion = capPromo
        self.UnitType = baseType
        self.BombardType = bombardType
        self.SpecialBombardUnitType = specBombardType        
    
    def setType(self, eType):
        self.UnitType = eType
        
    def setCapacityPromotedType(self, eType):
        self.CapacityPromotedType = eType
        
    def getSpecialBombardUnitType(self):
        return self.SpecialBombardUnitType
    
    def getBombardType(self):
        return self.BombardType
        
    def createBombard(self, pUnit):
        pPlayer = gc.getPlayer(pUnit.getOwner())
        iX = pUnit.getX()
        iY = pUnit.getY()
        iBombard = gc.getInfoTypeForString(self.BombardType)
        pBombard = pPlayer.initUnit(iBombard, iX, iY, UnitAITypes.UNITAI_ATTACK_AIR)
        
        for iPromotion in range(gc.getNumPromotionInfos()):
            if(pUnit.isHasPromotion(iPromotion)):
                # Exclude capacity promotions
                if(gc.getPromotionInfo(iPromotion).getType() in Config.bombardPromotionExclusions):
                    pass
                else:
                    pBombard.setHasPromotion(iPromotion, True)
    
    def checkForCapacityPromotion(self, pUnit, iPromotion):
        newUnit = pUnit
        if(gc.getPromotionInfo(iPromotion).getType() == self.CapacityPromotion):
            newUnit = UnitUtils.upgradeUnit(pUnit, gc.getInfoTypeForString(self.CapacityPromotedType))
        return newUnit
    
    def isValidBombard(self, pBombard):
        if(gc.getUnitInfo(pBombard.getUnitType()).getType() == self.BombardType):
            return True
        else:
            return False
        