## Colonize the Moon Mod Project
## Changes copyright Shin 2005
## (if any portion of this copyright should be found to be void, the remainder shall remain in full effect)
##
## CityUtils
#
# --Shin
# (aka. Belizan)

from CvPythonExtensions import *
from types import *
import random
import string
import cPickle
import Config

gc = CyGlobalContext()

def initCityDictionary(pCity):
    cityDict = {}
    pCity.setScriptData(cPickle.dumps(cityDict))
    
def getCityDictionary(pCity):
    if(pCity.getScriptData() != ""):
        cityDict = cPickle.loads(pCity.getScriptData())
    else:
        cityDict = {}
    return cityDict
    
def setCityDictionary(pCity, cityDict):
    pCity.setScriptData(cPickle.dumps(cityDict))
    
def getCityDictionaryValue(pCity, sValKey):
    cityDict = getCityDictionary(pCity)
    if(cityDict.has_key(sValKey)):
        return cityDict[sValKey]
    else:
        return 0
        
def setCityDictionaryValue(pCity, sValKey, oVal):
    cityDict = getCityDictionary(pCity)
    cityDict[sValKey] = oVal
    setCityDictionary(pCity, cityDict)
    
def increaseCityDictionaryValue(pCity, sValKey, iIncrement):
    cityDict = getCityDictionary(pCity)
    cityDict[sValKey] = cityDict[sValKey] + iIncrement
    setCityDictionary(pCity, cityDict)
    return cityDict[sValKey]
    
def removeCityDictionaryValue(pCity, sValKey):
    cityDict = getCityDictionary(pCity)
    if(cityDict.has_key(sValKey)):
        del cityDict[sValKey]
    setCityDictionary(pCity, cityDict)
    
def setArchitecture(pCity, sArchitecture):
    cityDict = getCityDictionary(pCity)
    cityDict["architecture"] = sArchitecture
    setCityDictionary(pCity, cityDict)
    
def getArchitecture(pCity):
    cityDict = getCityDictionary(pCity)
    if(cityDict.has_key("architecture")):
        return cityDict["architecture"]
    else:
        return "Default"
    
def clearRecentCombat(pCity):
    cityDict = getCityDictionary(pCity)
    cityDict["recentCombat"] = 0
    setCityDictionary(pCity, cityDict)
    
def pushRecentCombat(pCity):
    cityDict = getCityDictionary(pCity)
    if(cityDict.has_key("recentCombat")):
        iRC = cityDict["recentCombat"] + 1
    else:
        iRC = 1
    cityDict["recentCombat"] = iRC
    setCityDictionary(pCity, cityDict)
    return iRC
    
    
def makeExtraUnhealthy(pCity, iNewUnhealthiness):
    setCityDictionaryValue(pCity, "extraUnhealthiness", iNewUnhealthiness)
    iX = pCity.getX()
    iY = pCity.getY()        
    message = "City %s set to %d extra unhealthy" %(pCity.getName(), iNewUnhealthiness)
    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iX,iY,True,True)

def changeExtraUnhealthiness(pCity, iUnhealthinessChange):
    iNew = increaseCityDictionaryValue(pCity, "extraUnhealthiness", iUnhealthinessChange)
    iX = pCity.getX()
    iY = pCity.getY()        
    message = "City %s has had its extra unhealiness changed by %d to %d" %(pCity.getName(), iNewUnhealthiness, iNew)
    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iX,iY,True,True)
    

def removeExtraUnhealthy(pCity):
    removeCityDictionaryValue(pCity, "extraUnhealthiness")
    iX = pCity.getX()
    iY = pCity.getY()        
    message = "City %s extra unhealthiness removed" %(pCity.getName())
    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iX,iY,True,True)


def colorizeCityGridForPlot(pPlot, pPlayer):
    if(pPlot.isNone()):
        return
    if(pPlot.isPeak() or pPlot.isWater() or pPlot.getTerrainType() == gc.getInfoTypeForString("TERRAIN_DUSTSEA") or pPlot.getTerrainType() == gc.getInfoTypeForString("TERRAIN_CRATERRIM") or not pPlot.isRevealed(pPlayer.getTeam(), False)):
        CyEngine().addColoredPlotAlt(pPlot.getX(), pPlot.getY(), PlotStyles.PLOT_STYLE_BOX_OUTLINE, PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER, "COLOR_RED", .7)
        return
    iX = pPlot.getX()
    iY = pPlot.getY()
    sColor = "COLOR_CYAN"
    for x in range(-2, 3):
        for y in range(-2, 3):
            if(abs(x) + abs(y) != 4):
                pPlot = gc.getMap().plot(iX + x, iY + y)
                if(not pPlot.isNone()):
                    CyEngine().addColoredPlotAlt(pPlot.getX(), pPlot.getY(), PlotStyles.PLOT_STYLE_BOX_OUTLINE, PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER, sColor, .7)




def getBaseBuildingType(iBuilding):
    eBuilding = getBaseTag(iBuilding)
    return gc.getInfoTypeForString(eBuilding)
    
def getBaseTag(iBuilding):
    eBuilding = gc.getBuildingInfo(iBuilding).getType()
    iDX = string.rindex(eBuilding, '_') 
    eBuilding = eBuilding[0:iDX]
    return eBuilding

def getTagEncodedType(iBuilding):
    eBuilding = gc.getBuildingInfo(iBuilding).getType()
    iDX = string.rindex(eBuilding, '_') + 1
    eBuilding = eBuilding[iDX:]
    if (eBuilding in Config.TagEncodings):
        return eBuilding
    else:
        return "NONE"
        

# Currently assumes iTech is an untagged tech
def isSuchTaggedBuilding(iBuilding, lTags):
    eBuilding = gc.getBuildingInfo(iBuilding).getType()
    for eTag in lTags:
        sType = eBuilding + eTag
        if(gc.getInfoTypeForString(sType) != -1):
            return True
    return False

def getAllTaggedBuildings(iBuilding):
    lTags = []
    sTagBase = getTagBaseForBuilding(iBuilding)
    if(Config.DebugMode):
        message = "Tag Base found in get All Tags: %s" %(sTagBase)
        CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
    for eTag in Config.TagEncodings:
        sType = sTagBase + "_" + eTag
        if(gc.getInfoTypeForString(sType) != -1):
            lTags.append(eTag)
    return lTags

def getTagBaseForBuilding(iBuilding):
    eBuilding = gc.getBuildingInfo(iBuilding).getType()
    iDX = string.rindex(eBuilding, '_') + 1
    eTag = eBuilding[iDX:]
    if (eTag in Config.TagEncodings):
        if(Config.DebugMode):
            message = "Tag Base for Building(eTag is tag)--eTag: %s, iDX: %d, returnVal: %s, eBuilding: %s" %(eTag, iDX, eBuilding[0:iDX-1], eBuilding)
            CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
        return eBuilding[0:iDX-1]
    else:
        if(Config.DebugMode):
            message = "Tag Base fro Building(eTag is not tag)--eTag: %s, iDX: %d, returnVal: %s, eBuilding: %s" %(eTag, iDX, eBuilding, eBuilding)
            CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
        return eBuilding
