## Colonize the Moon Mod Project
## Changes copyright Shin 2005, 2006
## (if any portion of this copyright should be found to be void, the remainder shall remain in full effect)
##
## Config
#
# --Shin
# (aka. Belizan)

from CvPythonExtensions import *
from types import *
import random
import string
import cPickle
import Config
import Artillery
import TechEdge

#added
import UnitUtils
import CityUtils
import PlayerUtils
#####

gc = CyGlobalContext()

DebugMode = False

#***********************************************************************************************
##
## Configuration settings are all in this section...
##
## Format--
# Single pound lines indicate section or entry description.
### lines following a description indicate format
##
## Features are broken into three categories.  Main Designations start the configuration section
## followed by General and Specific features.  Specific feature settings offer settings specific
## to the moon mod.  General features are generic features which are being used to implement
## specific effects in the mod.
##
## Settings whose description ends with a * have not been implemented yet.
##
## Each section is separated by a divider of #s, followed by the section heading as follows:
#####################
### Main Designations
#***********************************************************************************************


# Tag Encodings (for Unit and Building tags)
### (TagSubStr, ...)
TagEncodings = ('MILITARY', 'COMMERCIAL', 'SCIENTIFIC', 'HAPPINESS')

# Non-military Units
### (UnitType, ...)
nonMilitaryUnits = ('UNIT_ENGINEER', 'UNIT_AI', 'UNIT_GENERAL', 'UNIT_SCIENTIST', 'UNIT_GREAT_ENGINEER', 'UNIT_LUNAR_SCOUT', 'UNIT_LUNAR_ROVER')

# Free food the AI when Isolation comes around*
AINumTurnsFreeFoodPerDifficultyOnIsolation = 5

####################
### General Features

### Buildings
#####
# One time XP awards OnBuildingBuilt
### Building : [XP]
def initOneTimeXPAwardsForBuildings():
    OneTimeXPAwardDict = {}
    OneTimeXPAwardDict["BUILDING_WAR_FOOTING"] = "5"
    #OneTimeXPAwardDict["BUILDING_BARRACKS"] = "4"
    setOneTimeXPAwardDict(OneTimeXPAwardDict)

# Units Linked to Buildings
### Building : Unit
def initUnitLinkedBuildings():
    UnitLinkedBuildingsDict = {}
    UnitLinkedBuildingsDict["BUILDING_REALLY_RESIDENT_AI"] = "UNIT_AI_AGENTS"
    #UnitLinkedBuildingsDict["BUILDING_BARRACKS"] = "UNIT_ENGINEER"
    setUnitLinkedBuildingsDict(UnitLinkedBuildingsDict)

# Building-generated Temporary Promotions*
### Building    Range   ProvidesP-closure     Promotion
def initBuildingGeneratedTempPromotions():
    BuildingGeneratedTempPromotionsDict = {}

# Resource-depending Buildings (Limiting buildings to production in cities which have specified resources in their city grid)*
### Building : (AndP, (Resource, ...))

# Building Upgrades*
### UpgradedBuilding : RequiredBaseBuilding
def initBuildingUpgrades():
    BuildingUpgradesDict = {}
    
# Building Exclusions (buildings which can not co-exist in the same city)*
### (PreventsConstructionP, (Building, ...))
def initBuildingExclusions():
    BuildingExclusionsDict = {}
    
# Resource-Utilizing Buildings (Buildings which produce yields based on resource usage in city grid)
### Building    Resource    (Yield tuple)   PerOccuranceP
def initResourceBuildings():
    ResourceBaseBuildingYieldsDict = {}
    ResourceBaseBuildingYieldsDict["BUILDING_HELIUM3_EXCHANGE"] = { "BONUS_HELIUM3": ((0, 0, (2, 0, 0)), True) }
    ResourceBaseBuildingYieldsDict["BUILDING_HELIUM3_EXCHANGE_COMMERCIAL"] = { "BONUS_HELIUM3": ((0, 0, (2, 0, 0)), True) }
    setResourceBaseBuildingYieldsDict(ResourceBaseBuildingYieldsDict)


### Improvements
#####
# Units Linked to Improvements*
### Improvement : Unit
def initUnitLinkedImprovements():
    UnitLinkedImprovementsDict = {}

# Improvement-generated Temporary Promotions*
### Improvement    Range   ProvidesP-closure     Promotion
def initImprovementGeneratedTempPromotions():
    ImprovementGeneratedTempPromotionsDict = {}
    
    
### Units
#####
# Unit-grenerated Temporary Promotions*
### UnitType    Range   ProvidesP-closure   Promotion
def initUnitGeneratedTempPromotions():
    UnitGeneratedTempPromotionsDict = {}

# Unit capture effects (does the unit capture another type of unit when it defeats it in combat)
### UnitType    CapturesP-closure
def initUnitCaptures():
    UnitCapturesDict = {}
    UnitCapturesDict["UNIT_LANCERS"] = ('combatclass == "UNITCOMBAT_TRACKED"', 'unittype')
    setUnitCapturesDict(UnitCapturesDict)
    
# Unit surrender effects (does the unit surrender (get captured) when it is defeated by another type of unit)
### UnitType    SurrendersP-closure     SurrenderedUnitType-closure
def initUnitSurrenders():
    UnitSurrendersDict = {}
    UnitSurrendersDict['UNIT_PISTOLMEN'] = ('dice.get(20, "Surrender Test") == 0', '"UNIT_PISTOLMEN"')
    setUnitSurrendersDict(UnitSurrendersDict)

# Unit leave salvage
### UnitType    Salvage-closure
def initSalvageable():
    SalvageableDict = {}
    Salvageable["UNIT_LUNAR_SCOUT"] = ""

### Terrain Attrition

# *
### {Terrain : AttritionP-closure   AttritionLevel-closure}
def initTerrainAttrition():
    TerrainAttritionDict = {}
    TerrainAttritionDict["TERRAIN_PEAK"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "1")   # Maria Basalt
    TerrainAttritionDict["TERRAIN_DUSTPLAINS"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "dice.get((distancecategory+1), 'dust plains attrition')")  # Dust Plains
    TerrainAttritionDict["TERRAIN_CRATERED"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "dice.get((distancecategory+1)/2, 'cratered attrition')")  # Cratered
    TerrainAttritionDict["TERRAIN_BAREROCK"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "dice.get(1, 'dust plains attrition') * dice.get((distancecategory+1)/2, 'bare rock attrition')")  # Bare Rock
    TerrainAttritionDict["TERRAIN_POLAR"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "dice.get(1, 'dust plains attrition') * dice.get((distancecategory+1)/2, 'polar attrition')")    # Polar
    TerrainAttritionDict["TERRAIN_DUSTSEA"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "dice.get((distancecategory+1) * 2, 'dust sea attrition')")   # Dust Sea
    TerrainAttritionDict["TERRAIN_DUSTSEA_WATER"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "dice.get((distancecategory+1) * 2, 'dust sea attrition')")    # Crater Rim
    TerrainAttritionDict["TERRAIN_MARIABASALT"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "dice.get(2, 'dust plains attrition')")    # ??
    TerrainAttritionDict["TERRAIN_MARIABASALT2"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "dice.get(2, 'dust plains attrition')")    # ??
    TerrainAttritionDict["TERRAIN_CRATERRIM"] = ("not isfriendlyterritory and dice.get(distancecategory+1, 'attrition terrain')", "dice.get(2, 'dust plains attrition') * dice.get(distancecategory+1, 'bare rock attrition')")    # ??
    setTerrainAttritionDict(TerrainAttritionDict)
    
# Survival Rating*
### 



### Tech Edge System

# Temporary promotions used by the Tech Edge system  (These are removed every turn and reassigned)
### (Promotion, ...)
techEdgePromotions = ("PROMOTION_TE_HEIGHTENED_CONTROL", "PROMOTION_TE_DEFENSE", "PROMOTION_TE_ADVTRAINING", "PROMOTION_TE_SATELLITE_RECON", "PROMOTION_TE_MAPS", "PROMOTION_TE_FUSION", "PROMOTION_TE_ADVANCED_TACTICS", "PROMOTION_TE_HEAL", "PROMOTION_TE_TRAINING", "PROMOTION_TE_COLONY_TACTICS", "PROMOTION_TE_ADVANCED_ARMOR", "PROMOTION_TE_FUTURISTIC_ARMOR")

# Tech Edge Triggers
### Technology : [TechEdgeTrigger, ...]
def initTechTriggers():
    techTriggerDict = {}
    
    #techTriggerDict["TECH_ISOLATION"]
    techTriggerDict["TECH_AIR_PURIFICATION"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_HEAL", "combatclass == 'UNITCOMBAT_FOOT'", "1")]
    techTriggerDict["TECH_LOW-G_TRAINING"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_TRAINING", "combatclass == 'UNITCOMBAT_FOOT'", "1", 2)]
    techTriggerDict["TECH_LOW-G_COMBAT"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_COLONY_TACTICS", "combatclass == 'UNITCOMBAT_HEAVY'", "1")]
    techTriggerDict["TECH_ADVANCED_ALLOYS"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_ADVANCED_ARMOR", "combatclass == 'UNITCOMBAT_TRACKED'", "1")]
    techTriggerDict["TECH_FUTURISTIC_ALLOYS"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_FUTURISTIC_ARMOR", "combatclass == 'UNITCOMBAT_TRACKED'", "2")]
    techTriggerDict["TECH_RADAR_SEISMOLOGY"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_ADVANCED_TACTICS", "combatclass == 'UNITCOMBAT_TRACKED'", "3")]
    techTriggerDict["TECH_SIMPLE_FUSION"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_FUSION", "combatclass == 'UNITCOMBAT_TRACKED'", "1", 2)]
    techTriggerDict["TECH_REGIONAL_EXPLORATION"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_MAPS", "combatclass == 'UNITCOMBAT_RECON'", "1")]
    techTriggerDict["TECH_SATELLITES"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_SATELLITE_RECON", "True", "1", 0)]
    techTriggerDict["TECH_LOW-G_WARFARE"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_ADVTRAINING", "unittype == 'UNIT_LUNAR_MARINES'", "1", 10)]
    techTriggerDict["TECH_HYPERCOMPRESSION"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_DEFENSE", "unittype == 'UNIT_LANCERS'", "1", 10)]
    techTriggerDict["TECH_AUTOMATION"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_TE_HEIGHTENED_CONTROL", "unittype == 'UNIT_RADIO_CAR'", "1", 10)]
    #techTriggerDict["TECH_HUNTING"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_SPEARS", "combatclass == 'UNITCOMBAT_MELEE'", "1")]
    #techTriggerDict["TECH_BRONZE_WORKING"] = [TechEdge.TechnologyTriggeredPromotion("PROMOTION_BRONZE_WEAPONRY", "combatclass != 'NONE' and combatclass != 'UNITCOMBAT_SIEGE' and combatclass != 'UNITCOMBAT_HELICOPTER' and combatclass != 'UNITCOMBAT_ARMOR'", "2")]
    setTechTriggerDict(techTriggerDict)


### Caravans*

# List of techs which grant caravan efficiency promotions*
### ( Tech, ...)
caravanFoodEfficiencyTechs = ()
caravanProdEfficiencyTechs = ()

# The efficiency progression (amount of res type gained per turn per caravan, indexed by number of efficiency upgrades researched)*
### ( fraction, fraction, ... )
caravanFoodEfficiencies = (1/4, 7/24, 1/3, 5/12, 1/2, 3/4, 1)
caravanProdEfficiencies = (1/4, 7/24, 1/3, 5/12, 1/2, 3/4, 1)

#####################
### Specific Features
# Great General Production Point requirements
GreatGeneralProductionBase = 150
General = "UNIT_GENERAL"

# Artillery settings
bombards = ('UNIT_BOMBARD')
bombardPromotionExclusions = ('PROMOTION_CAPACITY', 'PROMOTION_CAPACITY2', 'PROMOTION_CAPACITY3')

def initArtillery():
    # Establish artillery relationships
    artilleryDict = {}
    artilleryDict["UNIT_RANGED_ARTILLERY"] = Artillery.Artillery("UNIT_RANGED_ARTILLERY", "UNIT_BOMBARD", "SPECIALUNIT_BOMBARD", "UNIT_RANGED_ARTILLERY2", "PROMOTION_CAPACITY")
    artilleryDict["UNIT_RANGED_ARTILLERY2"] = Artillery.Artillery("UNIT_RANGED_ARTILLERY2", "UNIT_BOMBARD", "SPECIALUNIT_BOMBARD", "UNIT_RANGED_ARTILLERY3", "PROMOTION_CAPACITY2")
    artilleryDict["UNIT_RANGED_ARTILLERY3"] = Artillery.Artillery("UNIT_RANGED_ARTILLERY3", "UNIT_BOMBARD", "SPECIALUNIT_BOMBARD", "UNIT_RANGED_ARTILLERY4", "PROMOTION_CAPACITY3")
    artilleryDict["UNIT_RANGED_ARTILLERY4"] = Artillery.Artillery("UNIT_RANGED_ARTILLERY4", "UNIT_BOMBARD", "SPECIALUNIT_BOMBARD", "NONE", "NONE")
    setArtilleryDict(artilleryDict)
    

# Turrets
# Turret Unit Classes
### (UnitClass, ...)
TurretUnitClasses = ('UNITCLASS_BASETURRET', 'UNITCLASS_BIGTURRET', 'UNITCLASS_BIGGESTTURRET', 'UNITCLASS_SUPERTURRET', 'UNITCLASS_MONSTROUSTURRET', 'UNITCLASS_GARGANTUANTURRET')

# Turret Base Unit Class
### UnitClass
TurretBaseClass = 'UNITCLASS_BASETURRET'

# Food Caravans*
### Unit : FoodRating-closure
def initFoodCaravans():
    FoodCaravansDict = {}




#*********************************************************************************************
##
## Change nothing below this line!
##
#*********************************************************************************************
def getOneTimeXPAwardDict():
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    OneTimeXPAwardDict = gameDict["OneTimeXPAwardDict"]
    return OneTimeXPAwardDict
    
def setOneTimeXPAwardDict(OneTimeXPAwardDict):
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    gameDict["OneTimeXPAwardDict"] = OneTimeXPAwardDict
    gc.getGame().setScriptData(cPickle.dumps(gameDict))
    
def getUnitLinkedBuildingsDict():
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    UnitLinkedBuildingsDict = gameDict["UnitLinkedBuildingsDict"]
    return UnitLinkedBuildingsDict
    
def setUnitLinkedBuildingsDict(UnitLinkedBuildingsDict):
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    gameDict["UnitLinkedBuildingsDict"] = UnitLinkedBuildingsDict
    gc.getGame().setScriptData(cPickle.dumps(gameDict))
    
def getResourceBaseBuildingYieldsDict():
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    ResourceBaseBuildingYieldsDict = gameDict["ResourceBaseBuildingYieldsDict"]
    return ResourceBaseBuildingYieldsDict
    
def setResourceBaseBuildingYieldsDict(ResourceBaseBuildingYieldsDict):
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    gameDict["ResourceBaseBuildingYieldsDict"] = ResourceBaseBuildingYieldsDict
    gc.getGame().setScriptData(cPickle.dumps(gameDict))    


def getTerrainAttritionDict():
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    TerrainAttritionDict = gameDict["TerrainAttritionDict"]
    return TerrainAttritionDict
    
def setTerrainAttritionDict(TerrainAttritionDict):
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    gameDict["TerrainAttritionDict"] = TerrainAttritionDict
    gc.getGame().setScriptData(cPickle.dumps(gameDict))    


def getUnitCapturesDict():
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    UnitCapturesDict = gameDict["UnitCapturesDict"]
    return UnitCapturesDict
    
def setUnitCapturesDict(UnitCapturesDict):
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    gameDict["UnitCapturesDict"] = UnitCapturesDict
    gc.getGame().setScriptData(cPickle.dumps(gameDict))
    
def getUnitSurrendersDict():
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    UnitSurrendersDict = gameDict["UnitSurrendersDict"]
    return UnitSurrendersDict
    
def setUnitSurrendersDict(UnitSurrendersDict):
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    gameDict["UnitSurrendersDict"] = UnitSurrendersDict
    gc.getGame().setScriptData(cPickle.dumps(gameDict))

def getArtilleryDict():
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    artilleryDict = gameDict["artilleryDict"]
    return artilleryDict
    
def setArtilleryDict(artilleryDict):
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    gameDict["artilleryDict"] = artilleryDict
    gc.getGame().setScriptData(cPickle.dumps(gameDict))
    
def getTechTriggerDict():
    techTriggerDict = {}
    if(gc.getGame().getScriptData() != ""):
        gameDict = cPickle.loads(gc.getGame().getScriptData())
        if(gameDict.has_key("techTriggerDict")):
            techTriggerDict = gameDict["techTriggerDict"]
    return techTriggerDict
    
def setTechTriggerDict(techTriggerDict):
    gameDict = cPickle.loads(gc.getGame().getScriptData())
    gameDict["techTriggerDict"] = techTriggerDict
    gc.getGame().setScriptData(cPickle.dumps(gameDict))



########################################
#Added by Matthewv




def getCivBuildingUnblocked(iBuilding,pPlayer):
        bBlocked = False
        sType = CityUtils.getTagEncodedType(iBuilding)
        if(sType == "NONE"):
            #Test for one we care about
            eBuilding = gc.getBuildingInfo(iBuilding).getType()
            sType = eBuilding + "_MILITARY"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                    bBlocked = True
            sType = eBuilding + "_HAPPINESS"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                    bBlocked = True
            sType = eBuilding + "_COMMERCIAL"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                    bBlocked = True
            sType = eBuilding + "_SCIENTIFIC"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                    bBlocked = True
        elif(sType == "MILITARY"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                bBlocked = True
        elif(sType == "HAPPINESS"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                bBlocked = True
        elif(sType == "COMMERCIAL"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                bBlocked = True
        elif(sType == "SCIENTIFIC"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                bBlocked = True

        if (bBlocked == False):
            Unblocked = True
        else:
            Unblocked = False

            
        return Unblocked



def getCivUnitUnblocked(iUnit,pPlayer):
        bBlocked = False
        sType = UnitUtils.getTagEncodedType(iUnit)
        if(sType == "NONE"):
            #Test for one we care about
            eUnit = gc.getUnitInfo(iUnit).getType()
            sType = eUnit + "_MILITARY"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                    bBlocked = True
            sType = eUnit + "_COMMERCIAL"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                    bBlocked = True
            sType = eUnit + "_SCIENTIFIC"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                    bBlocked = True
        elif(sType == "MILITARY"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                bBlocked = True
        elif(sType == "COMMERCIAL"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                bBlocked = True
        elif(sType == "SCIENTIFIC"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                bBlocked = True



        if (bBlocked == False):
            Unblocked = True
        else:
            Unblocked = False

            
        return Unblocked

