## Colonize the Moon Mod Project
## Copyright Shin 2005
## (if any portion of this copyright should be found to be void, the remainder shall remain in full effect)
## 
## CvMoonEventManager
#
#
# Dance Card:
# Terrain Attrition
# Granted Promotions by Range from Improvement
# Granted Promotions by Range from Unit
# Granted Promotions by Range from Building
# Events--
#   Impacts
#   Meteorite Storms
#   Solar Flares
#   Bore Mine accidents
#   "Dust Storms"
#   Moonquakes
# Recruit from Earth  (Convert Merc Mod)
# Great Statesman -- Free Civic
# SATs/ASATs
# Supply Routes
# Salvage Creation
# New Goody Types--
#   Crash Site
#   Impact Crater
#   Salvage
#       Introduce Breakdowns (from Attition of mechanical units)
#
# ToDo:
# Cleanup BeginPlayerTurn/EndPlayerTurn issues
# Generalize unit-linked improvement references.
# Move Unit-linked improvement configuration info to Config.py
# Generalize Improvement-derived Promotions
# Move Improvement-derived promotion config info into Config.py
# Generalized building upgrades
# Generalized building class exclusions  (Only one of a set)
# Implement no unit within 1 square of major city power of National Alert
# Clean up files -- Make MegaImprovements.py, CustomTraits.py, etc. -- encapsulate tests
# Add/Remove Extra Unhealthiness from civic each turn (to catch new/lost or captured cities)
# Add to city display production value of resource dependent buildings in building list
# Add to main screen--gold production & tech progress, advisor&city screen--food, prod, commerce yields, tech chooser-res production additions for resource depending buildings
#
# ToDo(?):
# Grant radar benefit to allies
# limit radar benefit to certain unit classes
# limit One time XP bonus to certain unit types/classes
# Generalize Turrets (more.. if possible) (?)
#
# Cosmetics:
# Hide in CvTechChooser and TechSplash unaccessable (extra) buildings and units.
# Adjust messages to use icons of associated units
#
# UUs from unit class to unit types
# gc.getCivilizationInfo(pPlayer.getCivilizationType()).getCivilizationUnits(iUnitClass)
# gc.getGame().getActiveCivilizationType()
# The event hooks for the Python Moon demo Mod
# --Shin
# (aka. Belizan)


from CvPythonExtensions import *
from CvScreenEnums import *
import CvUtil
import CvScreensInterface
import CvDebugTools
import CvWBPopups
import PyHelpers
import Popup as PyPopup
import CvCameraControls
import CvTopCivs
import CvEventManager
import CvWorldBuilderScreen
import CvAdvisorUtils
import CvTechChooser

import sys
import cPickle

import UnitUtils
import CityUtils
import PlayerUtils
import ForensicsUtils
import DebugUtils
import Config
import Artillery
import Turrets
import TechEdge
import GameEvents
import Satellites
import TerrainAttrition
import Salvage

import CvHighlightEvents

gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
bCityPlacementMode = False

# globals
###################################################
class CvMoonEventManager(CvEventManager.CvEventManager):
    def __init__(self):
        # initialize base class
        self.parent = CvEventManager.CvEventManager
        self.parent.__init__(self)
        self.hilite = CvHighlightEvents.PlotHighlight()
        self.CustomEvents={
            5021 : ('EventPopup', self.__eventMoonModEventPopupApply, self.__eventMoonModEventPopupBegin),
            5022 : ('EventSignPopup', self.__eventMoonModAddSignEventPopupApply, self.__eventMoonModAddSignEventPopupBegin),
            5023 : ('EventSignPopup', self.__eventMoonModAddSiteEventPopupApply, self.__eventMoonModAddSiteEventPopupBegin),
        }
        
    def beginEvent( self, context, argsList=-1 ):
        'Begin Event'
        if(self.CustomEvents.has_key(context)):
            return self.CustomEvents[context][2](argsList)
        else:
            return self.parent.beginEvent(self, context, argsList)
            
    def applyEvent( self, argsList ):
        'Apply the effects of an event '
        context, playerID, netUserData, popupReturn = argsList

        if(self.CustomEvents.has_key(context)):
            entry = self.CustomEvents[context]
            return entry[1]( playerID, netUserData, popupReturn )   # the apply function
        else:
            return self.parent.applyEvent(self, argsList)
            
    def __eventMoonModEventPopupBegin(self, argsList):
        'Generic Popup Begin'
        #iX = pPlayer.getStartingPlot().getX()
        #iY = pPlayer.getStartingPlot().getY()
        message = "In __eventMoonModEventPopupBegin"
        CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
    
    def __eventMoonModEventPopupApply(self, playerID, netUserData, popupReturn):
        pNotification = netUserData[0]
        pNotification.apply(playerID, netUserData, popupReturn)
        
    def __eventMoonModAddSignEventPopupBegin(self, argsList):
        ' Add landmark sign popup '
        popup = PyPopup.PyPopup(5022, EventContextTypes.EVENTCONTEXT_ALL)
        popup.setUserData(argsList)
        popup.setHeaderString( "Add Marker..." )
        message = "What do you want your marker to say?"
        popup.setBodyString( message )
        popup.createEditBox( "" )
        popup.setEditBoxMaxCharCount( 15 )
        popup.launch(False, PopupStates.POPUPSTATE_QUEUED)
    
    def __eventMoonModAddSignEventPopupApply(self, playerID, netUserData, popupReturn):
        sCaption = str(popupReturn.getEditBoxString(0))
        pPlot = gc.getMap().plot(netUserData[0], netUserData[1])
        CyEngine().addSign(pPlot, playerID, sCaption)
        
    def __eventMoonModAddSiteEventPopupBegin(self, argsList):
        ' Add city marker popup '
        popup = PyPopup.PyPopup(5023, EventContextTypes.EVENTCONTEXT_ALL)
        popup.setUserData(argsList)
        popup.setHeaderString( "Place a City Site..." )
        message = "Enter city marker--"
        popup.setBodyString( message )
        popup.createEditBox( "City: " )
        popup.setEditBoxMaxCharCount( 21 )
        popup.launch(False, PopupStates.POPUPSTATE_QUEUED)
    
    def __eventMoonModAddSiteEventPopupApply(self, playerID, netUserData, popupReturn):
        sCaption = str(popupReturn.getEditBoxString(0))
        pPlot = gc.getMap().plot(netUserData[0], netUserData[1])
        CyEngine().addSign(pPlot, playerID, sCaption)
        PlayerUtils.addSiteToList(gc.getPlayer(playerID), pPlot)



    def onCivicsChanged(self, argsList):
        'New event triggered by a change in civics'
        lOld, lNew, iPlayer = argsList
        pPlayer = gc.getPlayer(iPlayer)
        
        if(lOld[6] != lNew[6]): # and lNew[6] == gc.getInfoTypeForString("CIVIC_NORMALCY")
            # Special Civic changed...
            #PlayerUtils.getPlayerDictionaryValue(pPlayer, "lastSpecial")
            PlayerUtils.setPlayerDictionaryValue(pPlayer, "lastSpecialCivic", gc.getGame().getGameTurn())
            
            # State of War
            if(lNew[6] == gc.getInfoTypeForString("CIVIC_WAR")):
                # Freeze Culture
                for iDX in range(pPlayer.getNumCities()):
                    pCity = pPlayer.getCity(iDX)
                    CityUtils.setCityDictionaryValue(pCity, "specialCivicCulture", pCity.getCulture(pCity.getOwner()))
            
            # National Emergency
            if(lNew[6] == gc.getInfoTypeForString("CIVIC_NATIONAL_EMERGENCY")):
                # Make Five Smallest Cities unhealthy
                dCities = {}
                for iDX in range(pPlayer.getNumCities()):
                    pCity = pPlayer.getCity(iDX)
                    iPop = pCity.getPopulation()
                    if(dCities.has_key(iPop)):
                        dCities[iPop].append(pCity)
                    else:
                        dCities[iPop] = []
                        dCities[iPop].append(pCity)
                        
                if(len(dCities) > 0):
                    lCitySizes = dCities.keys()
                    lCitySizes.sort()
                    # Get 5 smallest cities...
                    smallestCities = []
                    count = 0
                    iDX = 0
                    while count < 5 and iDX < len(lCitySizes):
                        lCities = dCities[lCitySizes[iDX-1]]
                                    
                        message = "Walking dCities: %s for lCitySizes: %s -- (%d)" %(str(dCities), str(lCitySizes), iDX)
                        #CyInterface().addMessage(0,True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
                        for pCity in lCities:
                            smallestCities.append(pCity)
                            count = count + 1
                        iDX = iDX + 1
                    for i in range(min((len(smallestCities), 5))):
                        pCity = smallestCities[i]
                        CityUtils.makeExtraUnhealthy(pCity, 1)
            
            if(lOld[6] == gc.getInfoTypeForString("CIVIC_NATIONAL_EMERGENCY")):
                for iDX in range(pPlayer.getNumCities()):
                    pCity = pPlayer.getCity(iDX)
                    CityUtils.removeExtraUnhealthy(pCity)
                iVal = PlayerUtils.getPlayerDictionaryValue(pPlayer, "specialCivicUnhappiness")
                pPlayer.changeExtraHappiness(iVal)
                PlayerUtils.removePlayerDictionaryValue(pPlayer, "specialCivicUnhappiness")
                    
            # National Alert
            if(lNew[6] == gc.getInfoTypeForString("CIVIC_NATIONAL_ALERT")):
                pHPlayer = PyPlayer(pPlayer.getID())
                iLOS = CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), "PROMOTION_SPECIAL_CIVIC_LOS")
                iMove = CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), "PROMOTION_SPECIAL_CIVIC_MOVE")
                for pUnit in pHPlayer.getUnitList():
                    pUnit.setHasPromotion(iLOS, True)
                    if(gc.getUnitInfo(pUnit.getUnitType()).getType() in Config.nonMilitaryUnits):       # None which I take to mean Non-Military
                        pUnit.setHasPromotion(iMove, True)
            
            if(lOld[6] == gc.getInfoTypeForString("CIVIC_NATIONAL_ALERT")):
                pHPlayer = PyPlayer(pPlayer.getID())
                iLOS = CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), "PROMOTION_SPECIAL_CIVIC_LOS")
                iMove = CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), "PROMOTION_SPECIAL_CIVIC_MOVE")
                for pUnit in pHPlayer.getUnitList():
                    pUnit.setHasPromotion(iLOS, False)
                    pUnit.setHasPromotion(iMove, False)
                    
                iVal = PlayerUtils.getPlayerDictionaryValue(pPlayer, "specialCivicUnhappiness")
                pPlayer.changeExtraHappiness(iVal)
                PlayerUtils.removePlayerDictionaryValue(pPlayer, "specialCivicUnhappiness")
                
            # End programmatic Civic effects
            message = "Set lastSpecialCivic to %d for %d" %(gc.getGame().getGameTurn(), iPlayer)
            #CyInterface().addMessage(0,True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
    

    def onGameStart(self, argsList):
        gameDict = {}
        gameDict["shortTermGameEvents"] = {}
        gameDict["longTermGameEvents"] = {}
        gc.getGame().setScriptData(cPickle.dumps(gameDict))
        Config.initOneTimeXPAwardsForBuildings()
        Config.initUnitLinkedBuildings()
        Config.initArtillery()
        Config.initUnitCaptures()
        Config.initUnitSurrenders()
        Config.initTechTriggers()
        Config.initResourceBuildings()
        Config.initTerrainAttrition()
        
        self.parent.onGameStart(self, argsList)
        
        for i in range(gc.getMAX_CIV_PLAYERS()):
            pPlayer = gc.getPlayer(i)
            if (pPlayer.isAlive()):
                PlayerUtils.initPlayerDictionary(pPlayer)
                GameEvents.prepNotificationQueue(pPlayer)
                PlayerUtils.setPlayerDictionaryValue(pPlayer, "gameEvents", {})
                
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                    pPlayer.setCivics(gc.getInfoTypeForString("CIVICOPTION_CHARTER"), gc.getInfoTypeForString("CIVIC_INDEPENDENT"))
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                    pPlayer.setCivics(gc.getInfoTypeForString("CIVICOPTION_CHARTER"), gc.getInfoTypeForString("CIVIC_ECONOMIC"))
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                    pPlayer.setCivics(gc.getInfoTypeForString("CIVICOPTION_CHARTER"), gc.getInfoTypeForString("CIVIC_SCIENTIFIC"))
                
                #initialSatScan = GameEvents.GameEvent((i,), (Satellites.Image, pPlayer.getStartingPlot().getX(), pPlayer.getStartingPlot().getY(), 5))
                #GameEvents.addEvent("0B", initialSatScan)
                # 1st turn of game is actually game turn -1, so do first notification by hand
                Satellites.StartingImage(pPlayer, (Satellites.Image, pPlayer.getStartingPlot().getX(), pPlayer.getStartingPlot().getY(), 5))
                
                #iX = pPlayer.getStartingPlot().getX()
                #iY = pPlayer.getStartingPlot().getY()
                #iWorker = CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_RANGED_ARTILLERY")
                #pPlayer.initUnit(iWorker, iX, iY, UnitAITypes.NO_UNITAI)
                #iWorker = CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_WORKER")
                #pPlayer.initUnit(iWorker, iX, iY, UnitAITypes.NO_UNITAI)
                
                #message = "Notifications: %s" %(PlayerUtils.getPlayerDictionaryValue(pPlayer, "notificationQueue"))
                #CyInterface().addMessage(pPlayer.getID(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
                
                #GameEvents.startNotifications(pPlayer)

    
    
    # Tech Edge
    # Unit-Linked Improvements
    # Turrets
    def onBeginPlayerTurn(self, argsList):
        'Called at the end of a players turn'
        iGameTurn, iPlayer = argsList
        pPlayer = gc.getPlayer(iPlayer)
        pHPlayer = PyPlayer(iPlayer)        
        pTeam = pHPlayer.getTeam()
        
        message = "Begin Player Turn %d" %(gc.getGame().getGameTurn())
        #CyInterface().addMessage(pPlayer.getID(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
        
        # Game Options
        #Signs
        #iLast = iPlayer - 1
        #if(iPlayer == gc.getBARBARIAN_PLAYER()):
        #    for i in range(gc.getMAX_CIV_PLAYERS()):
        #        pTmpPlayer = gc.getPlayer(i)
        #        if (pTmpPlayer.isAlive() and not iPlayer):
        #            iLast = i
        
        #if(iLast != -1):
        #    lSigns = PlayerUtils.getPlayerDictionaryValue("SignsList")
        #    if(lSigns != -1):
        #        for iSign in range(CyEngine().getNumSigns()):
        #            pSign = CyEngine().getSignByIndex(iSign)
        #            pSignPlot = pSign.getPlot()
        #            if(lSigns.count((pSignPlot.getX(), pSignPlot.getY())) > 0):
        #                CyEngine().removeSign(pSignPlot, -1)
        
        # Must change lSigns into a dictionary
        # Must check if lSigns has a caption registered, and register the sign's caption if absent


        #DebugUtils.debugStr("iImm1: %d, iImm2: %d" %(iImmigration1, iImmigration2))
            
        
        # Initialize this turn's Game Events
        GameEvents.prepNotificationQueue(pPlayer)
        
        # Economic Modifications
        ### Adjust for shipping routes, variable production buildings, and extra unhealithness
        
        # Tech Edge Triggers
        triggerCollection = TechEdge.getTriggersFor(pTeam)
        for pUnit in pHPlayer.getUnitList():
            for ePromotion in Config.techEdgePromotions:
                pUnit.setHasPromotion(gc.getInfoTypeForString(ePromotion), False)
                
            for key in triggerCollection.keys():
                triggerList = triggerCollection[key]
                precedenceOrder = {}
                for trigger in triggerList:
                    if(trigger.testApplicable(pUnit)):
                        precedenceOrder[trigger.getPrecedence(pUnit)] = trigger
                
                if(len(precedenceOrder.keys()) > 0):
                    precedenceOrder[max(precedenceOrder.keys())].applyPromotion(pUnit)
                    
        # Unit-Linked Improvements
        for pUnit in pHPlayer.getUnitList():
            if(gc.getUnitInfo(pUnit.getUnitType()).getType() == "UNIT_RADARSENTRY"):
                iImp = pUnit.plot().getImprovementType()
                eImp = "None"
                if(iImp != -1):
                    eImp = gc.getImprovementInfo(iImp).getType()
                    
                if (eImp != "IMPROVEMENT_RADARTOWER"):
                    pUnit.setDamage(100, True)                            
                    
            if(gc.getUnitInfo(pUnit.getUnitType()).getType() == "UNIT_AIRFIELD"):
                iImp = pUnit.plot().getImprovementType()
                eImp = "None"
                if(iImp != -1):
                    eImp = gc.getImprovementInfo(iImp).getType()
                    
                if (eImp != "IMPROVEMENT_AIRFIELD"):
                    pUnit.setDamage(100, True)
        
        # Auto-Upgrade Turrets
        for pUnit in pHPlayer.getUnitList():
            Turrets.upgradeTurret(pUnit)
            
        # onCivicsChanged notice support
        if(not pPlayer.isHuman()):
            lOrig = []
            for i in range (gc.getNumCivicOptionInfos()):
                lOrig.append(pPlayer.getCivics(i))
            PlayerUtils.setPlayerDictionaryValue(pPlayer, "AICivics", lOrig)
            
        # Special Civics
        iSpecialOpt = gc.getInfoTypeForString("CIVICOPTION_SPECIAL")
        iSpecial = pPlayer.getCivics(iSpecialOpt)
        if(iSpecial == gc.getInfoTypeForString("CIVIC_WAR")):
            # Freeze Culture
            for iDx in range(pPlayer.getNumCities()):
                pCity = pPlayer.getCity(iDx)
                iCulture = CityUtils.getCityDictionaryValue(pCity, "specialCivicCulture")
                pCity.setCulture(pCity.getOwner(), iCulture, True)
        
        # Special Civic Warnings
        iLast = PlayerUtils.getPlayerDictionaryValue(pPlayer, "lastSpecialCivic")
        iDuration = gc.getGame().getGameTurn() - iLast
        bWarning = False
        bPenalty = False
        if(iSpecial == gc.getInfoTypeForString("CIVIC_NATIONAL_EMERGENCY")):
            if(iDuration > 9):
                bPenalty = True
            elif(iDuration == 9):
                bWarning = True
                
        if(bWarning and pPlayer.isHuman()):
            if (pPlayer.canRevolution(0)):
                popupInfo = CyPopupInfo()
                popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_CHANGECIVIC)
                popupInfo.addPopup(iPlayer)
        if(bPenalty):
            if(pPlayer.isHuman()):
                if (pPlayer.canRevolution(0)):
                    popupInfo = CyPopupInfo()
                    popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_CHANGECIVIC)
                    popupInfo.addPopup(iPlayer)
            else:
                #lOrig = []
                #lNew = []
                #for i in range (gc.getNumCivicOptionInfos()):
                    #lOrig.append(pPlayer.getCivics(i))
                    #lNew.append(pPlayer.getCivics(i))
                #lNew[iSpecialOpt] = gc.getInfoTypeForString("CIVIC_NORMALCY")
                pPlayer.setCivics(iSpecialOpt, gc.getInfoTypeForString("CIVIC_NORMALCY"))
                #self.onCivicsChanged((lOrig, lNew, iPlayer))
        
        # Process Start of Turn Events
        GameEvents.handleEvents(pPlayer, True)
        GameEvents.startNotifications(pPlayer)
        
        self.parent.onBeginPlayerTurn(self,argsList)

    
    
    
    def onBeginGameTurn(self, argsList):
        iGameTurn = argsList[0]
        pMap = gc.getMap()
        
        for iX in range(pMap.getGridWidth()):
            for iY in range(pMap.getGridHeight()):
                pPlot = pMap.plot(iX, iY)
                if(pPlot.isCity()):
                    pCity = pPlot.getPlotCity()
                    CityUtils.clearRecentCombat(pCity)
        
        iMod = gc.getGameSpeedInfo(gc.getGame().getGameSpeedType()).getTrainPercent()
        iImmigration1 = int(25 * iMod / 100) - 2
        iImmigration2 = int(75 * iMod / 100) - 2
        if(iGameTurn == iImmigration1 or iGameTurn == iImmigration2):
            for i in range(gc.getMAX_CIV_PLAYERS()):
                pPlayer = gc.getPlayer(i)
                if (pPlayer.isAlive() and not pPlayer.isBarbarian()):
                    if(pPlayer.getCaptialCity().isNone()):
                        iX = pPlayer.getStartingPlot().getX()
                        iY = pPlayer.getStartingPlot().getY()
                    else:
                        iX = pPlayer.getCapitalCity().getX()
                        iY = pPlayer.getCapitalCity().getY()
                    iColonist = CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_COLONISTS")
                    pPlayer.initUnit(iColonist, iX, iY, UnitAITypes.NO_UNITAI)
        
        self.parent.onBeginGameTurn(self, argsList)

    
    
    def onBuildingBuilt(self, argsList):
        pCity = argsList[0]
        iBuildingType = argsList[1]
        
        pPlayer = gc.getPlayer(pCity.getOwner())
        eBuilding = gc.getBuildingInfo(iBuildingType).getType()
        iX = pCity.getX()
        iY = pCity.getY()
        
        # Biodomes and Bunkers
        if(iBuildingType == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT") or iBuildingType == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME")):
            CityUtils.setArchitecture(pCity.plot(), "Biodome")
            # check for Bunker (tests may be unnecessary, I might just be able to always remove them)
            if(pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER"))):
                pCity.setHasRealBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER"), 0)

            if(pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT"))):
                pCity.setHasRealBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT"), 0)
        
        if(iBuildingType == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT") or iBuildingType == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER")):
            CityUtils.setArchitecture(pCity.plot(), "Bunker")
            # check for Biodome (tests may be unnecessary, I might just be able to always remove them)
            if(pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME"))):
                pCity.setHasRealBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME"), 0)

            if(pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT"))):
                pCity.setHasRealBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT"), 0)
        
        
        # One Time XP Awards
        OneTimeXPAwardDict = Config.getOneTimeXPAwardDict()
        if(OneTimeXPAwardDict.has_key(eBuilding)):
            # grant award
            sXP = OneTimeXPAwardDict[eBuilding]
            iVal = eval(sXP)
            pPlot = pCity.plot()
            for iUnit in range(pPlot.getNumUnits()):
                pUnit = pPlot.getUnit(iUnit)
                if(pUnit.getOwner() == pPlayer.getID()):
                    pUnit.changeExperience(iVal, -1)
        
        
        # Unit Linked Buildings
        UnitLinkedBuildingsDict = Config.getUnitLinkedBuildingsDict()
        if(UnitLinkedBuildingsDict.has_key(eBuilding)):
            # init unit
            eUnit = UnitLinkedBuildingsDict[eBuilding]
            pPlayer.initUnit(gc.getInfoTypeForString(eUnit), iX, iY, UnitAITypes.NO_UNITAI)
        
        
        # Woodelf's Movie Patch
        #self.parent.onBuildingBuilt(self, argsList)
        game = CyGame()
        if ((not CyGame().isNetworkMultiPlayer()) and (pCity.getOwner() == CyGame().getActivePlayer()) and isWorldWonderClass(gc.getBuildingInfo(iBuildingType).getBuildingClassType())):
            # If this is a wonder...
            popupInfo = CyPopupInfo()
            #popupInfo.setButtonPopupType(ButtonPopupTypes.BUTTONPOPUP_PYTHON_SCREEN)
            #popupInfo.setData1(iBuildingType)
            #popupInfo.setData2(pCity.getID())
            #popupInfo.setData3(0)
            #popupInfo.setText(u"showWonderMovie")
            #popupInfo.addPopup(pCity.getOwner())
    
        CvAdvisorUtils.buildingBuiltFeats(pCity, iBuildingType)
    
        #if (not parent.__LOG_BUILDING):
        #    return
        #CvUtil.pyPrint('%s was finished by Player %d Civilization %s' 
        #    %(PyInfo.BuildingInfo(iBuildingType).getDescription(), pCity.getOwner(), gc.getPlayer(pCity.getOwner()).getCivilizationDescription(0)))


    
    def onCityAcquired(self, argsList):
        owner,playerType,pCity,bConquest,bTrade = argsList
        
        # Bunker and Biodome damage
        if(not bTrade):     
            if(CityUtils.getArchitecture(pCity.plot()) == "Biodome"):
                # Do horrible death and destruction
                iX = pCity.getX()
                iY = pCity.getY()
                message = "The City's biodome is destroyed in the capture!  Massive casualties ensue..."
                CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Buildings/Laboratory.dds',ColorTypes(7),iX,iY,True,True)
                iPop = pCity.getPopulation()
                iFood = pCity.getFood()
                pCity.changeFood( (-1 * iFood) )
                if(iPop > 0):
                    #size 0 Cities!  8)
                    pCity.setPopulation( iPop / 2 )
                    CityUtils.setArchitecture(pCity.plot(), "Default")
                else:
                    pCity.kill()
                    self.parent.onCityAcquired(self, argsList)
                    return None
            if(CityUtils.getArchitecture(pCity.plot()) == "Bunker"):
                # Do bad things
                iPop = pCity.getPopulation()
                if(iPop > 1):
                    pCity.setPopulation( iPop - 1 )
        
        
        # Leadership Trait Building Exclusions
        for iBuilding in range(gc.getNumBuildingInfos()):
            if(pCity.hasBuilding(iBuilding)):
                pPlayer = gc.getPlayer(pCity.getOwner())
                sType = CityUtils.getTagEncodedType(iBuilding)
                bCheckForRegular = False
                
                if(sType == "MILITARY"):
                    if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                        # check for regular version
                        bCheckForRegular = True
                elif(sType == "HAPPINESS"):
                    if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                        # check for regular version
                        bCheckForRegular = True
                elif(sType == "COMMERCIAL"):
                    if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                        # check for regular version
                        bCheckForRegular = True
                elif(sType == "SCIENTIFIC"):
                    if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                        # check for regular version
                        bCheckForRegular = True
                
                if(bCheckForRegular):
                    iBase = CityUtils.getBaseBuildingType(iBuilding)
                    if(iBase != -1):
                        pCity.setHasRealBuilding(iBuilding, 0)
                        pCity.setHasRealBuilding(iBase, 1)
                        iBuilding = iBase
                
                if(sType == "NONE" or bCheckForRegular):
                    iReplacement = -1
                    #Test for one we care about
                    eBuilding = gc.getUnitInfo(iBuilding).getType()
                    sType = eBuilding + "_MILITARY"
                    if(gc.getInfoTypeForString(sType) != -1):
                        if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                            iReplacement = gc.getInfoTypeForString(sType)
                    sType = eBuilding + "_HAPPINESS"
                    if(gc.getInfoTypeForString(sType) != -1):
                        if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                            iReplacement = gc.getInfoTypeForString(sType)
                    sType = eBuilding + "_COMMERCIAL"
                    if(gc.getInfoTypeForString(sType) != -1):
                        if(PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                            iReplacement = gc.getInfoTypeForString(sType)
                    sType = eBuilding + "_SCIENTIFIC"
                    if(gc.getInfoTypeForString(sType) != -1):
                        if(PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                            iReplacement = gc.getInfoTypeForString(sType)
                            
                    if(iReplacement != -1):
                        pCity.setHasRealBuilding(iBuilding, 0)
                        pCity.setHasRealBuilding(iReplacement, 1)
        
        
        CityUtils.initCityDictionary(pCity)
        Turrets.clearUpgradeTest(pCity)
        
        self.parent.onCityAcquired(self, argsList)

    
    
    def onCityBuilt(self, argsList):
        'City Built'
        pCity = argsList[0]
        CityUtils.initCityDictionary(pCity)
        CityUtils.initCityDictionary(pCity.plot())
        CityUtils.setArchitecture(pCity.plot(), "Default")
        Turrets.clearUpgradeTest(pCity)
        self.parent.onCityBuilt(self, argsList)
        

    
    def onCombatResult(self, argsList):
        'Combat Result'
        pWinner,pLoser = argsList
        
        if(pWinner.isMadeAttack()):
            # Test Unit Captures
            if(UnitUtils.checkUnitCaptures(pWinner, pLoser)):
                #gc.getUnitInfo(pWinner.getUnitType()).getArtInfo(pWinner.getOwner(), False).getButton()
                #gc.getUnitInfo(pWinner.getUnitType()).getButton()
                message = "Your %s has captured %s's %s" %(pWinner.getName(), gc.getPlayer(pLoser.getOwner()).getName(), pLoser.getName())
                CyInterface().addMessage(pWinner.getOwner(),True,len(message),message,'AS2D_RELIGION_CONVERT',0,gc.getUnitInfo(pWinner.getUnitType()).getButton(),ColorTypes(7),pLoser.getX(),pLoser.getY(),True,True)
            elif(UnitUtils.checkUnitSurrenders(pWinner, pLoser)):
                message = "%s's %s has surrendered!" %(gc.getPlayer(pLoser.getOwner()).getName(), pLoser.getName())
                CyInterface().addMessage(pWinner.getOwner(),True,len(message),message,'AS2D_RELIGION_CONVERT',0,gc.getUnitInfo(pLoser.getUnitType()).getButton(),ColorTypes(7),pWinner.getX(),pWinner.getY(),True,True)
                message = "Your %s has surrendered!" %(pLoser.getName())
                CyInterface().addMessage(pLoser.getOwner(),True,len(message),message,'AS2D_RELIGION_CONVERT',0,gc.getUnitInfo(pLoser.getUnitType()).getButton(),ColorTypes(7),pLoser.getX(),pLoser.getY(),True,True)
            pDefender = pLoser
            iXP = pLoser.defenseXPValue()
        else:
            pDefender = pWinner
            iXP = pLoser.attackXPValue()
        
        # Biodome damaged?
        pPlot = pDefender.plot()
        if(pPlot.isCity()):
            pCity = pPlot.getPlotCity()
            if(CityUtils.getArchitecture(pCity.plot()) == "Biodome"):
                # Let's see if it was damaged
                iRecentCombat = CityUtils.pushRecentCombat(pCity)
                dice = gc.getGame().getSorenRand()
                spread = 100
                roll = dice.get(spread, "Biodome Damage")
                if(roll < iRecentCombat):
                    # Damage incurred
                    iX = pCity.getX()
                    iY = pCity.getY()
                    message = "Your biodome was damaged in the attack!  Colonists fail to exhail the oxygen in their lungs fast enough while screaming to avoid explosive decompression as the Biodome collapses!"
                    CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Buildings/Laboratory.dds',ColorTypes(7),iX,iY,True,True)
                    if(pCity.getOwner() != pDefender.getOwner()):
                        message = "The %s biodome was damaged in the attack!  Colonists fail to exhail the oxygen in their lungs fast enough while screaming to avoid explosive decompression as the Biodome collapses!" %(pCity.getName())
                        CyInterface().addMessage(pDefender.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Buildings/Laboratory.dds',ColorTypes(7),iX,iY,True,True)
                    
                    iPop = pCity.getPopulation()
                    iFood = pCity.getFood()
                    pCity.setHasRealBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME"), 0)
                    pCity.setHasRealBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT"), 0)
                    pCity.changeFood( (-1 * iFood) )
                    if(iPop > 0):
                        #size 0 Cities!  8)
                        pCity.setPopulation( iPop / 2 )
                        CityUtils.setArchitecture(pCity.plot(), "Default")
                    else:
                        pCity.kill()
        
        
        # Great General Progress!
        # add 1 - pLoser.XPValue + 1 to the winner's General GP (+50% for Independents)
        pPlayer = gc.getPlayer(pWinner.getOwner())
        iSpread = iXP + pWinner.getLevel() + 1
        pDice = gc.getGame().getSorenRand()
        iRoll = pDice.get(iSpread, "Great General GP")
        if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
            iRoll = iRoll + (iRoll/2)
        
        iCount = PlayerUtils.addToCounter(pPlayer, "GreatGeneral", iRoll)
        
        # When General GP > numGenerals/3.0 * GP Game Speed Modifier * GP Production Constant (100)
        iGameSpeedMod = gc.getGameSpeedInfo(gc.getGame().getGameSpeedType()).getGreatPeoplePercent()        
        iBurden = (PlayerUtils.getCounter(pPlayer, "NumGenerals") + 3 * Config.GreatGeneralProductionBase) / 3.0
        iBurden = iBurden * iGameSpeedMod / 100.0
        
        iX = pWinner.getX()
        iY = pWinner.getY()
        message = "%d/%d (+%d)" %(iCount, iBurden, iRoll)
        #CyInterface().addMessage(pWinner.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iX,iY,True,True)
        
        if(iCount > iBurden):
            # numGenerals++; spawn general; reset General GP
            PlayerUtils.resetCounter(pPlayer, "GreatGeneral")
            PlayerUtils.addToCounter(pPlayer, "NumGenerals", 1)
            iX = pWinner.getX()
            iY = pWinner.getY()
            message = "A great general has emerged!"
            pPlayer.initUnit(gc.getInfoTypeForString(Config.General), iX, iY, UnitAITypes.NO_UNITAI)
            CyInterface().addMessage(pWinner.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(8),iX,iY,True,True)
        
        self.parent.onCombatResult(self, argsList)
        
    
    
    # For Economic Factions, add an extra 50% GP Merchant production
    # For Scientific Factions, add an extra 50% GP Scientist production
    # Validate Unit-Linked Building Relationship
    # Validate Unit-Linked Improvements Relationship
    def onEndPlayerTurn(self, argsList):
        iGameTurn, iPlayer = argsList
        pPlayer = gc.getPlayer(iPlayer)
        pHPlayer = PyPlayer(iPlayer)
        
        message = "End Player Turn %d" %(gc.getGame().getGameTurn())
        #CyInterface().addMessage(pPlayer.getID(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
        
        global bCityPlacementMode
        bCityPlacementMode = False
        #DebugUtils.debugStr("In City Placement Mode: %s" %(str(bCityPlacementMode)))
        
        # Trait GP Clean up
        if(PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
            # +50% Great Merchant GP Production
            for iDX in range(pPlayer.getNumCities()):
                pCity = pPlayer.getCity(iDX)
                iGP = gc.getInfoTypeForString("UNIT_MERCHANT")
                iRate = pCity.getGreatPeopleUnitRate(iGP)
                pCity.changeGreatPeopleUnitProgress(iGP, (iRate/2))
            
        if(PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
            # +50% Great Scientist GP Production
            for iDX in range(pPlayer.getNumCities()):
                pCity = pPlayer.getCity(iDX)
                iGP = gc.getInfoTypeForString("UNIT_SCIENTIST")
                iRate = pCity.getGreatPeopleUnitRate(iGP)
                pCity.changeGreatPeopleUnitProgress(iGP, (iRate/2))
        
        
        #Artillery Tests        
        for pUnit in pHPlayer.getUnitList():
            if(Artillery.isBombard(pUnit)):
                Artillery.testPropriety(pUnit)
        
        for pUnit in pHPlayer.getUnitList():
            if(Artillery.isArtillery(pUnit)):
                if(Artillery.hasRoom(pUnit)):
                    # Create unit
                    Artillery.createBombard(pUnit)
                    
        
        # Unit-Linked Improvements
        for pUnit in pHPlayer.getUnitList():
            if(gc.getUnitInfo(pUnit.getUnitType()).getType() == "UNIT_RADARSENTRY"):
                iImp = pUnit.plot().getImprovementType()
                eImp = "None"
                
                if(iImp != -1):
                    eImp = gc.getImprovementInfo(iImp).getType()
                    
                if (eImp != "IMPROVEMENT_RADARTOWER"):
                    pUnit.setDamage(100, True)
            
            if(gc.getUnitInfo(pUnit.getUnitType()).getType() == "UNIT_AIRFIELD"):
                iImp = pUnit.plot().getImprovementType()
                eImp = "None"
                
                if(iImp != -1):
                    eImp = gc.getImprovementInfo(iImp).getType()
                    
                if (eImp != "IMPROVEMENT_AIRFIELD"):
                    pUnit.setDamage(100, True)
        
        
        # Unit-Linked Buildings
        for iDX in range(pPlayer.getNumCities()):
            pCity = pPlayer.getCity(iDX)
            UnitLinkedBuildingsDict = Config.getUnitLinkedBuildingsDict()
            for eBuilding in UnitLinkedBuildingsDict.keys():
                if(gc.getInfoTypeForString(eBuilding) != -1):
                    if(pCity.hasBuilding(gc.getInfoTypeForString(eBuilding))):
                        pPlot = pCity.plot()
                        found = False
                        for iUnit in range(pPlot.getNumUnits()):
                            pUnit = pPlot.getUnit(iUnit)
                            if(gc.getUnitInfo(pUnit.getUnitType()).getType() == UnitLinkedBuildingsDict[eBuilding]):
                                found = True
                        if(not found):
                            pPlayer.initUnit(gc.getInfoTypeForString(UnitLinkedBuildingsDict[eBuilding]), pPlot.getX(), pPlot.getY(), UnitAITypes.NO_UNITAI)
        
        # Extra Unhealthiness support
        for iDX in range(pPlayer.getNumCities()):
            pCity = pPlayer.getCity(iDX)
            iExtraUnhealthiness = CityUtils.getCityDictionaryValue(pCity, "extraUnhealthiness")
            if(iExtraUnhealthiness != 0):
                if(iExtraUnhealthiness < 0):
                    iExtraHealthiness = -1 * iExtraUnhealthiness
                    iExtraUnhealthiness = 0
                else:
                    iExtraHealthiness = 0
                iGoodHealth = pCity.goodHealth()
                iBadHealth = pCity.badHealth(False)
                iAdjBadHealth = iBadHealth + iExtraUnhealthiness
                iAdjGoodHealth = iGoodHealth + iExtraHealthiness
                iFoodCalc = min((iGoodHealth - iBadHealth, 0))
                iFoodReal = min(((iGoodHealth + iExtraHealthiness) - (iBadHealth + iExtraUnhealthiness)), 0)
                iFoodDelta = iFoodReal - iFoodCalc
                
                iFoodDifferenceOrig = pCity.foodDifference(True)
                iFoodDifference = iFoodDifferenceOrig + iFoodDelta
                
                if(iFoodDelta != 0):
                    if((iFoodDifference < 0) and (iFoodDifferenceOrig < 0)):                            
                        # All comes from food.
                        pCity.changeFood(iFoodDelta)
                    elif((iFoodDifference > 0) and (iFoodDifferenceOrig <0)):
                        if(pCity.isFoodProduction()):
                            pCity.changeFood(iFoodDifferenceOrig * -1)
                            pCity.changeProduction(iFoodDifference)
                        else:
                            pCity.changeFood(iFoodDelta)
                    elif((iFoodDifference < 0) and (iFoodDifferenceOrig > 0)):
                        if(pCity.isFoodProduction()):
                            pCity.changeProduction(-1 * iFoodDifferenceOrig)
                            pCity.changeFood(iFoodDifference)
                        else:
                            pCity.changeFood(iFoodDelta)
                    elif((iFoodDifference > 0) and (iFoodDifferenceOrig > 0)):
                        if(pCity.isFoodProduction()):
                            pCity.changeProduction(iFoodDelta)
                        else:
                            pCity.changeFood(iFoodDelta)
        
        
        # Resource-Linked Buildings
        dResourceLinkedBuildings = Config.getResourceBaseBuildingYieldsDict()
        for iDX in range(pPlayer.getNumCities()):
            #message = "Checking buildings in city %s" %(pCity.getName())
            #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
                                    
            pCity = pPlayer.getCity(iDX)
            for iBuilding in range(gc.getNumBuildingInfos()):
                if(pCity.hasBuilding(iBuilding)):
                    eBuilding = gc.getBuildingInfo(iBuilding).getType()
                    
                    #message = "Found building %s" %(eBuilding)
                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)

                    if(dResourceLinkedBuildings.has_key(eBuilding)):
                        
                        #message = "Found resource dependent building %s" %(eBuilding)
                        #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
            
                        dDependencies = dResourceLinkedBuildings[eBuilding]
                        # ResourceBaseBuildingYieldsDict["BUILDING_HELIUM3_EXCHANGE"] = { "BONUS_HELIUM3": ((0, 0, (0, 0, 2)), True) }
                        for sResource in dDependencies.keys():
                            tYieldBonus = dDependencies[sResource]
                            iBonus = gc.getInfoTypeForString(sResource)
                            bPerResource = tYieldBonus[1]
                            # Get resource count
                            iOrigX = pCity.plot().getX()
                            iOrigY = pCity.plot().getY()
                            iWorkRadius = 2
                            iCount = 0
                            # Written this way in case I want to change the algorithm later
                            for iX in range (-2, 3):
                                for iY in range (-2, 3):
                                    if(abs(iY) + abs(iX) <= iWorkRadius + 1):
                                        x = iOrigX + iX
                                        y = iOrigY + iY
                                        pPlot = gc.getMap().plot(x,y)
                                        if(pPlot.getBonusType(pPlayer.getTeam()) == iBonus):
                                            
                                            #bRes = pPlot.isPlotGroupConnectedBonus(pPlayer.getID(), iBonus)
                                            #bBNet = pPlot.isBonusNetwork(pPlayer.getTeam())
                                            #message = "Resource at %d, %d returns %s and %s" %(x, y, str(bRes), str(bBNet))
                                            #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
            
                                            # This gaurantees that only one city gets credit for each He3, even with overlapping zones
                                            if(pPlot.getWorkingCity().getID() == pCity.getID()):
                                                #if(pPlot.isBeingWorked()):
                                                #if(pPlot.isPlotGroupConnectedBonus(pPlayer.getID(), iBonus)):  # This will return true if any bonus of that type is connected, and that stile has a route to the network
                                                
                                                # Verify that the bonus is on the player's network
                                                if(pPlot.isBonusNetwork(pPlayer.getTeam())):
                                                    iImp = pPlot.getImprovementType()
                                                    if(iImp != -1):
                                                        if(gc.getImprovementInfo(iImp).isImprovementBonusMakesValid(iBonus)):
                                                            iCount = iCount + 1
                            
                        
                            # Add bonus
                            if(iCount > 0):
                                iFood = tYieldBonus[0][0]
                                iProd = tYieldBonus[0][1]
                                tCommerce = tYieldBonus[0][2]
                                iGold = tCommerce[0]
                                iRes = tCommerce[1]
                                iCult = tCommerce[2]
                                if(bPerResource):
                                    iFood = iFood * iCount
                                    iProd = iProd * iCount
                                    iGold = iGold * iCount
                                    iRes = iRes * iCount
                                    iCult = iCult * iCount
                                
                                if(iFood != 0):
                                    iFood = iFood + ((iFood * pCity.getYieldRateModifier(0)) / 100)
                                    #message = "Adding %d food to city %s" %(iFood, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    pCity.changeFood(iFood)
                                if(iProd != 0):
                                    iProd = iProd + ((iProd * pCity.getYieldRateModifier(1)) / 100)
                                    #message = "Adding %d production to city %s" %(iProd, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    pCity.changeProduction(iProd)
                                if(iGold != 0):
                                    iGold = iGold + ((iGold * pCity.getCommerceRateModifier(0)) / 100)
                                    #message = "Adding %d gold 'to' city %s" %(iGold, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    pPlayer.changeGold(iGold)
                                if(iRes != 0):
                                    iRes = iRes + ((iRes * pCity.getCommerceRateModifier(1)) / 100)
                                    #message = "Adding %d research 'to' city %s" %(iRes, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    gc.getTeam(pPlayer.getTeam()).changeResearchProgress(pPlayer.getCurrentResearch(), iRes, pPlayer.getID())
                                if(iCult != 0):
                                    iCult = iCult + ((iCult * pCity.getCommerceRateModifier(2)) / 100)
                                    #message = "Adding %d culture to city %s" %(iCult, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    pCity.changeCulture(pPlayer.getID(), iCult, True)
    
    
        
        # Civics
        # onCivicsChanged support
        if(not pPlayer.isHuman()):
            lOrig = PlayerUtils.getPlayerDictionaryValue(pPlayer, "AICivics")
            lNew = []
            bChanged = False
            for i in range (gc.getNumCivicOptionInfos()):
                lNew.append(pPlayer.getCivics(i))
                if(lNew[i] != lOrig[i]):
                    bChanged = True
            if(bChanged == True):
                self.onCivicsChanged((lOrig, lNew, iPlayer))
        
                
        # Special Civics Implementation
        iSpecial = gc.getInfoTypeForString("CIVICOPTION_SPECIAL")
        iSpecial = pPlayer.getCivics(iSpecial)
        iLast = PlayerUtils.getPlayerDictionaryValue(pPlayer, "lastSpecialCivic")
        iDuration = gc.getGame().getGameTurn() - iLast
        if(iSpecial == gc.getInfoTypeForString("CIVIC_NATIONAL_EMERGENCY")):                
            if(iDuration >= 10):
                pPlayer.changeExtraHappiness(-2)
                PlayerUtils.increasePlayerDictionaryValue(pPlayer, "specialCivicUnhappiness", 2)
        elif(iSpecial == gc.getInfoTypeForString("CIVIC_WAR")):
            # Freeze Culture
            for iDx in range(pPlayer.getNumCities()):
                pCity = pPlayer.getCity(iDx)
                iCulture = CityUtils.getCityDictionaryValue(pCity, "specialCivicCulture")
                pCity.setCulture(pCity.getOwner(), iCulture, True)
        elif(iSpecial == gc.getInfoTypeForString("CIVIC_FUNDAMENTALISM")):
            # Nothing yet
            pass
        elif(iSpecial == gc.getInfoTypeForString("CIVIC_NATIONAL_ALERT")):
            if(iDuration >= 5):
                pPlayer.changeExtraHappiness(-1)
                PlayerUtils.increasePlayerDictionaryValue(pPlayer, "specialCivicUnhappiness", 1)
        elif(iSpecial == gc.getInfoTypeForString("CIVIC_GENETIC_SCREENING")):
            # Nothing yet
            pass
            
        #Testing
        #lPS = {}
        #for i in range(gc.getMAX_CIV_PLAYERS()):
        #    ptPlayer = gc.getPlayer(i)
        #    if (ptPlayer.isAlive()):
        #        lPS[i] = PlayerUtils.getPlayerDictionaryValue(ptPlayer, "lastSpecialCivic")
        pHeadSelectedCity = pPlayer.getCapitalCity()
        iExtraUnhealthiness = CityUtils.getCityDictionaryValue(pHeadSelectedCity, "extraUnhealthiness")
        if(iExtraUnhealthiness < 0):
            iExtraHealthiness = -1 * iExtraUnhealthiness
            iExtraUnhealthiness = 0
        else:
            iExtraHealthiness = 0
        iGoodHealth = pHeadSelectedCity.goodHealth()
        iBadHealth = pHeadSelectedCity.badHealth(False)
        iAdjBadHealth = iBadHealth + iExtraUnhealthiness
        iAdjGoodHealth = iGoodHealth + iExtraHealthiness
        iFoodCalc = min((iGoodHealth - iBadHealth, 0))
        iFoodReal = min(((iGoodHealth + iExtraHealthiness) - (iBadHealth + iExtraUnhealthiness)), 0)
        iFoodDelta = iFoodReal - iFoodCalc
        message = "Good: %d(%d), Bad: %d(%d), Food Calc: %d, Food Real: %d(%d), dF: %d" %(iGoodHealth, iAdjGoodHealth, iBadHealth, iAdjBadHealth, iFoodCalc, iFoodReal, iExtraUnhealthiness, iFoodDelta)
        iX = pPlayer.getCapitalCity().getX()
        iY = pPlayer.getCapitalCity().getY()
        #message = "LSC Timer: %s" %(str(lPS))
        #CyInterface().addMessage(pPlayer.getID(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iX,iY,True,True)
        
        
        # Process End of Turn Events
        GameEvents.handleEvents(pPlayer, False)
        
        # AI Rest from Attrition Mode
        if(not pPlayer.isHuman()):
            for pUnit in pHPlayer.getUnitList():
                #if(UnitUtils.hasBeenAttrited(pUnit) and pUnit.getDamage() > 50):
                #   if over 90 just rest here
                if(pUnit.getDamage() > 90):
                    pGroup = pUnit.getGroup()
                    #pGroup.pushMission(MissionTypes.MISSION_HEAL, 0, 0, 0, False, False, MissionAITypes.NO_MISSIONAI, pUnit.plot(), pUnit)
                #   else:
                #       find nearest home points (10)
                #       assign mission path to them
                #       get turns required
                #       pick shortest
        
        # Game Options
        iGrid = PlayerUtils.getGameOption(pPlayer, "CityGrid")
        if(iGrid < 1):
            CyEngine().clearColoredPlots(PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER)
        else:
            PlayerUtils.colorizeCityPlots(pPlayer)
        
        # Testing
        if(Config.DebugMode):
            if(pPlayer.getID() == 0):
                pBarbarian = gc.getPlayer(gc.getBARBARIAN_PLAYER())
                iMarine =  CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_LUNAR_MARINES")
                iX = pPlayer.getStartingPlot().getX()
                iY = pPlayer.getStartingPlot().getY()
                #pUnit = pBarbarian.initUnit(iMarine, iX+2, iY+2, UnitAITypes.NO_UNITAI)
                #gameDict = cPickle.loads(gc.getGame().getScriptData())
                #gameDict["testID"] = pUnit.getID()
                #gameDict["testX"] = iX + 1
                #gameDict["testY"] = iY - 5
                #gc.getGame().setScriptData(cPickle.dumps(gameDict))
                #UnitUtils.AddMoveTo(pUnit, iX+1, iY-5)
                #pGroup = pUnit.getGroup()
                #pGroup.pushMoveToMission(iX+1, iY-5)
                
            #UnitUtils.ResetUnitOrders(pPlayer)
            if(pPlayer.getID() == gc.getBARBARIAN_PLAYER()):
                gameDict = cPickle.loads(gc.getGame().getScriptData())
                if(gameDict.has_key("testID")):
                    iID = gameDict["testID"]
                    iX = gameDict["testX"]
                    iY = gameDict["testY"]
                    pBarbarian = gc.getPlayer(gc.getBARBARIAN_PLAYER())
                    pUnit = pBarbarian.getUnit(iID)
                    pGroup = pUnit.getGroup()
                    #setUnitDictionaryValue(pUnit, "StandingOrders", "pGroup.pushMoveToMission(%d, %d)" %(iX, iY))
                    pGroup.pushMoveToMission(iX, iY)
                    #setUnitDictionaryValue(pUnit, "ClearStandingOrders", "pUnit.getPathEndTurnPlot().at(%d, %d)" %(iX, iY))
                    if(pGroup.getPathEndTurnPlot().at(iX, iY)):
                        message = "Test Unit has reached destination this turn." 
                        CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
                    else:
                        pMission = pGroup.getMissionFromQueue(0)
                        
                        message = "Test Unit at %d, %d (Mission: %d, %d/%d, [%d %d %d])." %(pUnit.getX(), pUnit.getY(), pMission.eMissionType, pMission.iPushTurn, gc.getGame().getGameTurn(), pMission.iData1, pMission.iData2, pMission.iFlags)
                        CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
        
                
                
        self.parent.onEndPlayerTurn(self, argsList)
        
    
    def onKbdEvent(self, argsList):
        eventType,key,mx,my,px,py = argsList
        if ( eventType == self.EventKeyDown ):
            iKey = int(key)
            if(self.bCtrl and iKey == InputTypes.KB_K):
                pPlayer = gc.getActivePlayer()
                iGrid = PlayerUtils.getGameOption(pPlayer, "CityGrid")
                message = "Handling ctrl-k"
                #CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
                if(iGrid < 1):
                    iGrid = 1
                    PlayerUtils.setGameOption(pPlayer, "CityGrid", iGrid)
                    PlayerUtils.colorizeCityPlots(pPlayer)
                else:
                    iGrid = 0
                    PlayerUtils.setGameOption(pPlayer, "CityGrid", iGrid)
                    CyEngine().clearColoredPlots(PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER)
                return 1
            if(self.bCtrl and iKey == InputTypes.KB_P):
                global bCityPlacementMode
                pPlot = CyInterface().getMouseOverPlot()
                if(self.bShift):
                    bCityPlacementMode = not bCityPlacementMode
                    DebugUtils.debugStr("In City Placement Mode: %s" %(str(bCityPlacementMode)))
                    if(bCityPlacementMode):
                        message = "Switching to City Placement Mode..."
                        CyInterface().addMessage(gc.getActivePlayer(),True,len(message),message,'',0,'Art/Interface/Buttons/actions/Recon.dds',ColorTypes(8),pPlot.getX(),pPlot.getY(),True,True)
                    else:
                        message = "Leaving City Placement Mode..."
                        CyInterface().addMessage(gc.getActivePlayer(),True,len(message),message,'',0,'Art/Interface/Buttons/actions/Recon.dds',ColorTypes(8),pPlot.getX(),pPlot.getY(),True,True)
                else:
                    if(bCityPlacementMode):
                        pPlayer = gc.getActivePlayer()
                        if(pPlot.isPeak() or pPlot.isWater() or pPlot.getTerrainType() == gc.getInfoTypeForString("TERRAIN_DUSTSEA") or pPlot.getTerrainType() == gc.getInfoTypeForString("TERRAIN_CRATERRIM") or not pPlot.isRevealed(pPlayer.getTeam(), False)):
                            pass
                        else:
                            bErase = False
                            for iSign in range(CyEngine().getNumSigns()):
                                pSign = CyEngine().getSignByIndex(iSign)
                                pSignPlot = pSign.getPlot()
                                if(pSignPlot.at(pPlot.getX(), pPlot.getY()) and pSign.getPlayerType() == pPlayer.getID()):
                                    bErase = True
                                    PlayerUtils.removeSiteFromList(pPlayer, pPlot)
                                    CyEngine().removeSign(pSignPlot, pPlayer.getID())
                                    break
                            if(bErase == False):
                                self.__eventMoonModAddSiteEventPopupBegin((pPlot.getX(), pPlot.getY(), pPlayer.getID()))
                    else:
                        pPlayer = gc.getActivePlayer()
                        iShowMoves = PlayerUtils.getGameOption(pPlayer, "ShowMoves")
                        if(iShowMoves < 1): 
                            iShowMoves = 1
                            PlayerUtils.setGameOption(pPlayer, "ShowMoves", iShowMoves)
                            #pUnit = CyInterface().getHeadSelectedUnit()
                            #if(not pUnit.isNone()):
                            #    pass    
                            if(CyInterface().getHeadSelectedUnit()):
                                if(not CyInterface().getHeadSelectedUnit().isNone()):
                                    self.hilite.onUpdate((1,1))
                        else:
                            iShowMoves = 0
                            PlayerUtils.setGameOption(pPlayer, "ShowMoves", iShowMoves)
                            self.hilite.hiliteOff()
                return 1
            if(self.bCtrl and iKey == InputTypes.KB_M):
                pPlayer = gc.getActivePlayer()
                pPlot = CyInterface().getMouseOverPlot()
                bErase = False
                for iSign in range(CyEngine().getNumSigns()):
                    pSign = CyEngine().getSignByIndex(iSign)
                    pSignPlot = pSign.getPlot()
                    if(pSignPlot.at(pPlot.getX(), pPlot.getY()) and pSign.getPlayerType() == pPlayer.getID()):
                        bErase = True
                        #PlayerUtils.removeSignFromList(pPlayer, pPlot)
                        CyEngine().removeSign(pSignPlot, pPlayer.getID())
                        break
                if(bErase == False):
                    self.__eventMoonModAddSignEventPopupBegin((pPlot.getX(), pPlot.getY(), pPlayer.getID()))
                    #ForensicsUtils.getInfoOnObject(CyEngine())
                #    CyEngine().addLandmarkPopup(pPlot)
                #    PlayerUtils.addSignToList(pPlayer, pPlot)
                return 1
            if(self.bCtrl and iKey == InputTypes.KB_N):
                if(Config.DebugMode):
                    pPlot = CyInterface().getMouseOverPlot()
                    for iSign in range(CyEngine().getNumSigns()):
                        pSign = CyEngine().getSignByIndex(iSign)
                        pSignPlot = pSign.getPlot()
                        if(pSignPlot.at(pPlot.getX(), pPlot.getY())):
                            message = "Sign '%s' owned by player %d" %(pSign.getCaption(), pSign.getPlayerType())
                            #ForensicsUtils.getInfoOnObject(pSign)
                            CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
                            CyEngine().removeSign(pSignPlot, -1)
                    return 1
                
        return self.parent.onKbdEvent(self, argsList)
        
        
    def onImprovementBuilt(self, argsList):
        'Improvement Built'
        iImprovement, iX, iY = argsList
        eImp = gc.getImprovementInfo(iImprovement).getType()
        if(eImp == "IMPROVEMENT_RADARTOWER"):
            map = gc.getMap()
            pPlot = map.plot(iX, iY)
            iPlayer = pPlot.getOwner()
            iSentry = CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_RADARSENTRY")
            gc.getPlayer(iPlayer).initUnit(iSentry, iX, iY, UnitAITypes.NO_UNITAI)
            
            # Do radar sweep PROMOTION_RADAR
            iRadar = CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), "PROMOTION_RADAR")
            for idX in range(-2,3):
                for idY in range(-2,3):
                    pPlot = gc.getMap().plot(iX + idX, iY + idY)
                    if(pPlot.isNone()):
                        pass
                    else:
                        for iUnit in range(pPlot.getNumUnits()):
                            pUnit = pPlot.getUnit(iUnit)
                            if(pUnit.getOwner() == iPlayer):
                                pUnit.setHasPromotion(iRadar, True)
        
        if(eImp == "IMPROVEMENT_AIRFIELD"):
            map = gc.getMap()
            pPlot = map.plot(iX, iY)
            iPlayer = pPlot.getOwner()
            iSentry = CvUtil.findInfoTypeNum(gc.getUnitInfo, gc.getNumUnitInfos(), "UNIT_AIRFIELD")
            gc.getPlayer(iPlayer).initUnit(iSentry, iX, iY, UnitAITypes.NO_UNITAI)
            
        self.parent.onImprovementBuilt(self,argsList)



    # Improvement Derived Promotions
    def onUnitCreated(self, argsList):
        'Unit Completed'
        pUnit = argsList[0]
        if(gc.getUnitInfo(pUnit.getUnitType()).getType() == "UNIT_RADARSENTRY"):
            UnitUtils.setLastXY(pUnit)
        
        if(gc.getUnitInfo(pUnit.getUnitType()).getType() == "UNIT_AIRFIELD"):
            UnitUtils.setLastXY(pUnit)
        
        iX = pUnit.getX()
        iY = pUnit.getY()
        # Do radar sweep PROMOTION_RADAR
        iRadar = CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), "PROMOTION_RADAR")
        for idX in range(-2,3):
            for idY in range(-2,3):
                pPlot = gc.getMap().plot(iX + idX, iY + idY)
                if(pPlot.isNone()):
                    pass
                else:
                    iImp = pPlot.getImprovementType()
                    eImp = "None"
                    if(iImp != -1):
                        eImp = gc.getImprovementInfo(iImp).getType()
                        
                    if (eImp == "IMPROVEMENT_RADARTOWER"):
                        if(pUnit.getOwner() == pPlot.getOwner()):
                            pUnit.setHasPromotion(iRadar, True)
                            break
                            
        # Special Civic Support Code
        iSpecial = gc.getInfoTypeForString("CIVICOPTION_SPECIAL")        
        if(iSpecial == gc.getInfoTypeForString("CIVIC_NATIONAL_ALERT")):
            iLOS = CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), "PROMOTION_SPECIAL_CIVIC_LOS")
            iMove = CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), "PROMOTION_SPECIAL_CIVIC_MOVE")
            pUnit.setHasPromotion(iLOS, True)
            if(pUnit.getUnitCombatType == -1):       # None which I take to mean Non-Military
                pUnit.setHasPromotion(iMove, True)
                
        self.parent.onUnitCreated(self,argsList)
        
        
        

    def onUnitKilled(self, argsList):
        'Unit Killed'
        pUnit, iAttacker = argsList
        UnitUtils.setUnitKilled(pUnit)
        
        self.parent.onUnitKilled(self, argsList)




    def onUnitLost(self, argsList):
        'Unit Lost'
        pUnit = argsList[0]
        
        # Create salvage?
        if(UnitUtils.getUnitDictionaryValue(pUnit, "attritionDeath") == True):
            Salvage.testCreateSalvage(pUnit, False)
        
        if(UnitUtils.getUnitKilled(pUnit) == True):
            pass
        elif(gc.getUnitInfo(pUnit.getUnitType()).getType() == "UNIT_RADARSENTRY"):
            pPlot = UnitUtils.getLastPlot(pUnit)
            iImp = pPlot.getImprovementType()
            eImp = "None"
            
            if(iImp != -1):
                eImp = gc.getImprovementInfo(iImp).getType()
            
            if (eImp == "IMPROVEMENT_RADARTOWER"):
                #ImprovementTypes.NO_IMPROVEMENT
                pPlot.setImprovementType(ImprovementTypes.NO_IMPROVEMENT)
                
        elif(gc.getUnitInfo(pUnit.getUnitType()).getType() == "UNIT_AIRFIELD"):
            pPlot = UnitUtils.getLastPlot(pUnit)
            iImp = pPlot.getImprovementType()
            eImp = "None"
            
            if(iImp != -1):
                eImp = gc.getImprovementInfo(iImp).getType()
            
            if (eImp == "IMPROVEMENT_AIRFIELD"):
                #ImprovementTypes.NO_IMPROVEMENT
                pPlot.setImprovementType(ImprovementTypes.NO_IMPROVEMENT)
        
        self.parent.onUnitLost(self, argsList)


        
        
    def onUnitMove(self, argsList):
        'unit move'
        pPlotBase,pUnit = argsList
        
        UnitUtils.setLastXY(pUnit)
        
        # Game Options
        pPlayer = gc.getPlayer(pUnit.getOwner())
        iShowMoves = PlayerUtils.getGameOption(pPlayer, "ShowMoves")
        if(iShowMoves == 1):
            self.hilite.onUnitMove(argsList)
        
        # Only do Terrain Attrition if the Map Option for it is set to On
        if(gc.getMap().getCustomMapOption(3) == 0):
            TerrainAttrition.testAttrition(pUnit)
        
        # Update Radar Coverage
        iX = pUnit.getX()
        iY = pUnit.getY()
        iRadar = CvUtil.findInfoTypeNum(gc.getPromotionInfo, gc.getNumPromotionInfos(), "PROMOTION_RADAR")
        pUnit.setHasPromotion(iRadar, False)
        for idX in range(-2,3):
            for idY in range(-2,3):
                pPlot = gc.getMap().plot((iX + idX), (iY + idY))
                
                if(pPlot.isNone()):
                    pass
                else:
                    iImp = pPlot.getImprovementType()
                    eImp = "None"
                    if(iImp != -1):
                        eImp = gc.getImprovementInfo(iImp).getType()
                        
                    if (eImp == "IMPROVEMENT_RADARTOWER"):
                        if(pUnit.getOwner() == pPlot.getOwner()):
                            pUnit.setHasPromotion(iRadar, True)
        
        self.parent.onUnitMove(self, argsList)
        
        
        
        
    def onUnitPromoted(self, argsList):
        'Unit Promoted'
        pUnit, iPromotion = argsList
        if(Artillery.isArtillery(pUnit)):
            newUnit = Artillery.checkForCapacityPromotion(pUnit, iPromotion)
            if(newUnit.movesLeft() > 0):
                CyInterface().selectUnit(newUnit, True, False, False)
        self.parent.onUnitPromoted(self, argsList)


    def onUnitSelected(self, argsList):
        'Unit Selected'
        pUnit = argsList[0]
        pPlayer = gc.getPlayer(pUnit.getOwner())
        
        # Game Options
        iGrid = PlayerUtils.getGameOption(pPlayer, "CityGrid")
        if(iGrid < 1):
            CyEngine().clearColoredPlots(PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER)
        else:
            PlayerUtils.colorizeCityPlots(pPlayer)
            
        paPlayer = gc.getActivePlayer()
        iShowMoves = PlayerUtils.getGameOption(paPlayer, "ShowMoves")
        if(iShowMoves == 1):
            self.hilite.onUnitSelected(argsList)
        
        ###*******************************************
        ### DEBUG TESTING CODE
        if(Config.DebugMode):
            lCommands = []
            for i in range(gc.getNumCommandInfos()):
                lCommands.append(str(gc.getCommandInfo(i).getType()) + "(" + str(i) + ")")
            
            pCity = paPlayer.getCapitalCity()
            #ForensicsUtils.getInfoOnObject(pCity)
            
            message = "Commands: %s" %(str(lCommands))
            #CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
            
            pAction = gc.getActionInfo(7) # Heal
            message = "CommandData: %d, CommandType: %d, InterfaceModeType: %d, MissionData: %d, MissionType: %d, IsConfirmCommand: %s, IsVisible: %s" %(pAction.getCommandData(), pAction.getCommandType(), pAction.getInterfaceModeType(), pAction.getMissionData(), pAction.getMissionType(), str(pAction.isConfirmCommand()), str(pAction.isVisible()))
            #CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
            
            pMission = gc.getMissionInfo(pAction.getMissionType())
            message = "Mission: %s, (%d), Waypoint: %s, isBuild: %s, isTarget: %s" %(pMission.getType(), pMission.getTime(), pMission.getWaypoint(), str(pMission.isBuild()), str(pMission.isTarget()))
            #CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
            
            pGroup = pUnit.getGroup()
            if(pGroup.isNone()):
                message = "Wah wah waaahhh...."
                CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
            
            pPlayer = gc.getPlayer(pUnit.getOwner())
            iX = pPlayer.getCapitalCity().getX()
            iY = pPlayer.getCapitalCity().getY()
            #pGroup.pushMoveToMission(iX, iY)   # This works
            #pGroup.pushMission(pAction.getMissionType(), 0, 0, 0, False, False, MissionAITypes.NO_MISSIONAI, pUnit.plot(), pUnit)    # This doesn't
            #pGroup.pushMission(MissionTypes.MISSION_HEAL, 0, 0, 0, False, False, MissionAITypes.NO_MISSIONAI, pUnit.plot(), pUnit)   # This works
            #pUnit.doCommand(pAction.getCommandType(), 7, pAction.getMissionData())  # Nor this
            #pUnit.NotifyEntity(7)   # Or This
            
            iC = pGroup.getLengthMissionQueue()
            message = "Len: %d" %(iC)
            #message = "Mission: %s, (%d), Waypoint: %s, isBuild: %s, isTarget: %s" %(pMission.getType(), pMission.getTime(), pMission.getWaypoint(), str(pMission.isBuild()), str(pMission.isTarget()))
            
            if(pGroup.getLengthMissionQueue() > 0):
                pPlot = pGroup.getPathEndTurnPlot()
                message = "Len: %d (%d, %d)" %(iC, pPlot.getX(), pPlot.getY())
                
                if(not pPlot.isNone()):
                    CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
            
                    #CyEngine().addColoredPlotAlt(pPlot.getX(), pPlot.getY(), PlotStyles.PLOT_STYLE_TARGET, PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_REVEALED_PLOTS, "COLOR_RED", .5)
            
            #CyEngine().clearColoredPlots(PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER)
            for iDX in range(pPlayer.getNumCities()):
                pCity = pPlayer.getCity(iDX)
                iX = pCity.getX()
                iY = pCity.getY()
                for x in range(-2, 3):
                    for y in range(-2, 3):
                        pPlot = gc.getMap().plot(iX + x, iY + y)
                        message = "Testing %d, %d" %(pPlot.getX(), pPlot.getY())
                        #CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
                        if(not pPlot.isNone()):
                            if(pPlot.getWorkingCity().getID() == pCity.getID()):
                                message = "Colorizing %d, %d" %(pPlot.getX(), pPlot.getY())
                                #CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
                                #CyEngine().addColoredPlotAlt(pPlot.getX(), pPlot.getY(), PlotStyles.PLOT_STYLE_BOX_FILL, PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_REVEALED_PLOTS, "COLOR_RED", .5)
                                #CyEngine().addColoredPlotAlt(pPlot.getX(), pPlot.getY(), PlotStyles.PLOT_STYLE_BOX_OUTLINE, PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER, "COLOR_WHITE", .7)
                            
            
            #ForensicsUtils.getInfoOnObject(PlotStyles)
        
        self.parent.onUnitSelected(self, argsList)
        
        
    def onUpdate(self, argsList):
        pPlayer = gc.getActivePlayer()
        iShowMoves = PlayerUtils.getGameOption(pPlayer, "ShowMoves")
        if(iShowMoves == 1):
            self.hilite.onUpdate(argsList)
        
        iGrid = PlayerUtils.getGameOption(pPlayer, "CityGrid")
        if(iGrid == 1):
            PlayerUtils.colorizeCityPlots(pPlayer)
            
        global bCityPlacementMode
        if(bCityPlacementMode):
            PlayerUtils.colorizeCitySites(pPlayer)
            CityUtils.colorizeCityGridForPlot(CyInterface().getMouseOverPlot(), pPlayer)
                
    def onMouseEvent(self, argsList):
        pPlayer = gc.getActivePlayer()
        iShowMoves = PlayerUtils.getGameOption(pPlayer, "ShowMoves")
        if(iShowMoves == 1):
            self.hilite.onMouseEvent(argsList)
            
        global bCityPlacementMode
        if(bCityPlacementMode):
            CyEngine().clearColoredPlots(PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER)
            iGrid = PlayerUtils.getGameOption(pPlayer, "CityGrid")
            if(iGrid == 1):
                PlayerUtils.colorizeCityPlots(pPlayer)
            pPlot = CyInterface().getMouseOverPlot()
            if(pPlot.isPeak() or pPlot.isWater() or pPlot.getTerrainType() == gc.getInfoTypeForString("TERRAIN_DUSTSEA") or pPlot.getTerrainType() == gc.getInfoTypeForString("TERRAIN_CRATERRIM") or not pPlot.isRevealed(pPlayer.getTeam(), False)):
                pass
            else:
                bErase = False
                for iSign in range(CyEngine().getNumSigns()):
                    pSign = CyEngine().getSignByIndex(iSign)
                    pSignPlot = pSign.getPlot()
                    if(pSignPlot.at(pPlot.getX(), pPlot.getY()) and pSign.getPlayerType() == pPlayer.getID()):
                        bErase = True
                        PlayerUtils.removeSiteFromList(pPlayer, pPlot)
                        CyEngine().removeSign(pSignPlot, pPlayer.getID())
                        break
                if(bErase == False):
                    self.__eventMoonModAddSiteEventPopupBegin((pPlot.getX(), pPlot.getY(), pPlayer.getID()))
        
        
        
        
        
            