## Colonize the Moon Mod Project
## Changes copyright Shin 2005
## (if any portion of this copyright should be found to be void, the remainder shall remain in full effect)
##
## CvMoonGameUtils
#
# Biodome
# Mega-improvement.  Determines city architecture.
#	Stores 25% of food after growth
#	+2 health
#	+3 happiness
#	+1 food in all squares with at least 2
#	+1 commerce in all squares with at least 3
#	Removes all defensive bonuses from the city
#	Can be destroyed by hostile troops
#	If destroyed, results in loss of  population and all food
#
# Bunker
# Mega-improvement.  Determines city architecture.
#	Stores 25% of food after growth
#	+1 happiness
#	+2 production
#	+25% defensive bonus
#	+1 production in all squares with at least 3
#	Removes happiness bonus from Biodome terrain improvement
#	Can be breached by hostile fire
#	If breached, results in loss of 1 population and defensive bonuses
#
# --Shin
# (aka. Belizan)

import CvUtil
from CvPythonExtensions import *
from types import *
import CvGameUtils
import CvEventInterface
import Popup as PyPopup
import UnitUtils
import CityUtils
import PlayerUtils
import TechUtils
import Config
import Turrets
import cPickle

# globals
gc = CyGlobalContext()


class CvMoonGameUtils(CvGameUtils.CvGameUtils):
    "Miscellaneous game functions"
    def __init__(self):
        self.parent = CvGameUtils.CvGameUtils
        self.parent.__init__(self)

    # Leadership trait based civics
    def cannotDoCivic(self,argsList):
        ePlayer = argsList[0]
        eCivic = argsList[1]
        
        pPlayer = gc.getPlayer(ePlayer)
        #excludedCivics = (gc.getInfoTypeForString("CIVIC_INDEPENDENT"), gc.getInfoTypeForString("CIVIC_ECONOMIC"), gc.getInfoTypeForString("CIVIC_SCIENTIFIC"))
        #if(eCivic in excludedCivics):
            #return False
            #return True
        if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT") and eCivic == gc.getInfoTypeForString("CIVIC_INDEPENDENT")):
            return True
        if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC") and eCivic == gc.getInfoTypeForString("CIVIC_ECONOMIC")):
            return True
        if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC") and eCivic == gc.getInfoTypeForString("CIVIC_SCIENTIFIC")):
            return True
        
        if(ePlayer == 0):
            iX = 0
            iY = 0        
            message = "Checking %s" %(gc.getCivicInfo(eCivic).getType())
            #CyInterface().addMessage(0,True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iX,iY,True,True)
            
        if(eCivic == gc.getInfoTypeForString("CIVIC_NORMALCY")):
            return False
        iSpecial = gc.getInfoTypeForString("CIVICOPTION_SPECIAL")
        if(gc.getCivicInfo(eCivic).getCivicOptionType() == iSpecial):
            if(pPlayer.getCivics(iSpecial) != gc.getInfoTypeForString("CIVIC_NORMALCY") and pPlayer.getCivics(iSpecial) != eCivic):
                return True
            
            if(eCivic == gc.getInfoTypeForString("CIVIC_WAR") and pPlayer.getCivics(iSpecial) != eCivic):
                if(gc.getTeam(pPlayer.getTeam()).getAtWarCount(True) > 0):          #True to ignoreMinors
                    return False
                else:
                    return True
            
            iLast = PlayerUtils.getPlayerDictionaryValue(pPlayer, "lastSpecialCivic")
            iRemaining = gc.getGame().getGameTurn() - iLast
            if(iLast == -1):
                return False
            if(eCivic == gc.getInfoTypeForString("CIVIC_NATIONAL_ALERT")):
                if(iRemaining > 19 or pPlayer.getCivics(iSpecial) == eCivic):
                    return False
                else:
                    return True
            elif(iRemaining > 49 or pPlayer.getCivics(iSpecial) == eCivic):
                return False
            else:
                return True
        
        return False

    # Tagged Unit construction
    def cannotTrain(self,argsList):
        pCity = argsList[0]
        iUnit = argsList[1]
        bContinue = argsList[2]
        bTestVisible = argsList[3]
        
        bBlocked = False
        pPlayer = gc.getPlayer(pCity.getOwner())
        
        # Turrets
        if(Turrets.cannotTrain(iUnit, pCity)):
            bBlocked = True
        
        # Leadership Traits
        sType = UnitUtils.getTagEncodedType(iUnit)
        if(sType == "NONE"):
            #Test for one we care about
            eUnit = gc.getUnitInfo(iUnit).getType()
            sType = eUnit + "_MILITARY"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                    bBlocked = True
            sType = eUnit + "_COMMERCIAL"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                    bBlocked = True
            sType = eUnit + "_SCIENTIFIC"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                    bBlocked = True
        elif(sType == "MILITARY"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                bBlocked = True
        elif(sType == "COMMERCIAL"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                bBlocked = True
        elif(sType == "SCIENTIFIC"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                bBlocked = True
        
        return bBlocked
        
       
    # Tagged Tech Research Restrictions       
    def cannotResearch(self,argsList):
        iPlayer = argsList[0]
        iTech = argsList[1]
        bTrade = argsList[2]
        
        bBlocked = False
        pPlayer = gc.getPlayer(iPlayer)
        
        # Leadership Traits
        sType = TechUtils.getTagEncodedType(iTech)
        if(sType == "NONE"):
            #Test for one we care about
            eTech = gc.getTechInfo(iTech).getType()
            sType = eTech + "_MILITARY"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                    bBlocked = True
            sType = eTech + "_COMMERCIAL"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                    bBlocked = True
            sType = eTech + "_SCIENTIFIC"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                    bBlocked = True
        elif(sType == "MILITARY"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                bBlocked = True
        elif(sType == "COMMERCIAL"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                bBlocked = True
        elif(sType == "SCIENTIFIC"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                bBlocked = True
                
        return bBlocked
    
    
    # Tagged Building construction
    def cannotConstruct(self,argsList):
        pCity = argsList[0]
        iBuilding = argsList[1]
        bContinue = argsList[2]
        bTestVisible = argsList[3]
        bIgnoreCost = argsList[4]
        
        bBlocked = False
        pPlayer = gc.getPlayer(pCity.getOwner())
        
        # Check Biodomes and Bunkers
        if(pPlayer.isHuman()):
            if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME")):
                #Test for Bunker
                if(pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT"))):
                    bBlocked = True
                        
            if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER")):
                #Test for Bunker
                if(pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT"))):
                    bBlocked = True
                        
            if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT")):
                if(not (pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME")))):
                    bBlocked = True
                    
            if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT")):
                if(not (pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER")))):
                    bBlocked = True
        else:
            iMil = PlayerUtils.getFlavorValue(pPlayer, "FLAVOR_MILITARY")
            iGrow = PlayerUtils.getFlavorValue(pPlayer, "FLAVOR_GROWTH")
            iSpread = iMil + iGrow + 2
            pDice = gc.getGame().getSorenRand()
            iRoll = pDice.get(iSpread, "AI Architecture Decision Rand")
            if(iRoll < iMil + 1):
                # Bunkers ok
                if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME")):
                    bBlocked = True
                        
                if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER")):
                    #Test for Bunker
                    if(pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT"))):
                        bBlocked = True
                            
                if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT")):
                    if(not (pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME")))):
                        bBlocked = True
                        
                if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT")):
                    bBlocked = True
            else:
                # Biodomes ok
                if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME")):
                    if(pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT"))):
                        bBlocked = True
                        
                if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER")):
                    bBlocked = True
                            
                if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT")):
                    bBlocked = True
                        
                if(iBuilding == gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BIODOME_REFIT")):
                    if(not (pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER_REFIT")) or pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_CITY_ARCHITECTURE_BUNKER")))):
                        bBlocked = True
        
        iX = pCity.getX()
        iY = pCity.getY()
        message = "GameUtils--%d" %(iBuilding)
        #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Buildings/Laboratory.dds',ColorTypes(7),iX,iY,True,True)
        
        # Leadership trait restrictions
        sType = CityUtils.getTagEncodedType(iBuilding)
        if(sType == "NONE"):
            #Test for one we care about
            eBuilding = gc.getBuildingInfo(iBuilding).getType()
            sType = eBuilding + "_MILITARY"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                    bBlocked = True
            sType = eBuilding + "_HAPPINESS"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                    bBlocked = True
            sType = eBuilding + "_COMMERCIAL"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                    bBlocked = True
            sType = eBuilding + "_SCIENTIFIC"
            if(gc.getInfoTypeForString(sType) != -1):
                if(PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                    bBlocked = True
        elif(sType == "MILITARY"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                bBlocked = True
        elif(sType == "HAPPINESS"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
                bBlocked = True
        elif(sType == "COMMERCIAL"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_ECONOMIC")):
                bBlocked = True
        elif(sType == "SCIENTIFIC"):
            if(not PlayerUtils.hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
                bBlocked = True
        
        return bBlocked
    
    
    def isActionRecommended(self,argsList):
        pUnit = argsList[0]
        iAction = argsList[1]
        
        gameDict = cPickle.loads(gc.getGame().getScriptData())
        if(gameDict.has_key("testID")):
            iID = gameDict["testID"]
            if pUnit.getID() == iID:
                pass
                #if(iAction != 
                #iX = gameDict["testX"]
                #iY = gameDict["testY"]
        return None


    def cannotHandleAction(self,argsList): 
        pPlot = argsList[0]
        iAction = argsList[1]
        bTestVisible = argsList[2]
        
        pAction = gc.getActionInfo(iAction)
        pMission = gc.getMissionInfo(pAction.getMissionType())
        
        try:
            eMission = pMission.getType()
        except:
            eMission = "NONE"
            
        if(eMission == "NONE"):
            message = "Action: (%d)%s" %(iAction, pAction.getButton())
            #CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
        else:
            message = "Action: %d, Mission: %s" %(iAction, eMission)
            #CyInterface().addMessage(0,True,len(message),message,'',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),0,0,True,True)
        
        return None 