## Colonize the Moon Mod Project
## Changes copyright Shin 2006
## (if any portion of this copyright should be found to be void, the remainder shall remain in full effect)
##
## EconomicUtils
#
# Support functions for the various economic modifying features put into the game (by me)
#
# getRealXXX -- Actual total XXX
# getDeltaXXX -- Required delta XXX
#
# XXX is one of:
# Food
# Production
# Gold
# Research
# Culture
#
# --Shin
# (aka. Belizan)

from CvPythonExtensions import *
from types import *
import random
import string
import cPickle
import Config

gc = CyGlobalContext()


# Resource-linked Buildings
# Supply Routes
# Extra unhealthiness
def getExtraFoodForCity(pCity):
    iDeltaFood = 0
    # Resource-linked Buildings
    for iBuilding in range(gc.getNumBuildingInfos()):
        if(pCity.hasBuilding(iBuilding)):
            eBuilding = gc.getBuildingInfo(iBuilding).getType()
            if(dResourceLinkedBuildings.has_key(eBuilding)):
                dDependencies = dResourceLinkedBuildings[eBuilding]
                # ResourceBaseBuildingYieldsDict["BUILDING_HELIUM3_EXCHANGE"] = { "BONUS_HELIUM3": ((0, 0, (0, 0, 2)), True) }
                for sResource in dDependencies.keys():
                    tYieldBonus = dDependencies[sResource]
                    # Test for Food
                    if(tYieldBonus[0][0] == 0):
                        continue
                    iBonus = gc.getInfoTypeForString(sResource)
                    bPerResource = tYieldBonus[1]
                    # Get resource count
                    iOrigX = pCity.plot().getX()
                    iOrigY = pCity.plot().getY()
                    iWorkRadius = 2
                    iCount = 0
                    # Written this way in case I want to change the algorithm later
                    for iX in range (-2, 3):
                        for iY in range (-2, 3):
                            if(abs(iY) + abs(iX) <= iWorkRadius + 1):
                                x = iOrigX + iX
                                y = iOrigY + iY
                                pPlot = gc.getMap().plot(x,y)
                                if(pPlot.getBonusType(pPlayer.getTeam()) == iBonus):
                                    # This gaurantees that only one city gets credit for each He3, even with overlapping zones
                                    if(pPlot.getWorkingCity().getID() == pCity.getID()):
                                        # Verify that the bonus is on the player's network
                                        if(pPlot.isBonusNetwork(pPlayer.getTeam())):
                                            iImp = pPlot.getImprovementType()
                                            if(iImp != -1):
                                                if(gc.getImprovementInfo(iImp).isImprovementBonusMakesValid(iBonus)):
                                                    iCount = iCount + 1
                    # Add bonus
                    if(iCount > 0):
                        iFood = tYieldBonus[0][0]
                        if(bPerResource):
                            iFood = iFood * iCount
                        if(iFood != 0):
                            iDeltaFood = iDeltaFood + iFood
                        if(iFood > 0):
                            iDeltaFood = iDeltaFood + ((iFood * pCity.getYieldRateModifier(0)) / 100)
    
    # Extra Unhealthiness
    iExtraUnhealthiness = CityUtils.getCityDictionaryValue(pCity, "extraUnhealthiness")
    if(iExtraUnhealthiness != 0):
        if(iExtraUnhealthiness < 0):
            iExtraHealthiness = -1 * iExtraUnhealthiness
            iExtraUnhealthiness = 0
        else:
            iExtraHealthiness = 0
        iGoodHealth = pCity.goodHealth()
        iBadHealth = pCity.badHealth(False)
        iAdjBadHealth = iBadHealth + iExtraUnhealthiness
        iAdjGoodHealth = iGoodHealth + iExtraHealthiness
        iFoodCalc = min((iGoodHealth - iBadHealth, 0))
        iFoodReal = min(((iGoodHealth + iExtraHealthiness) - (iBadHealth + iExtraUnhealthiness)), 0)
        iDeltaFood = iFoodReal - iFoodCalc
        
        
    # Supply Routes (Validated elsewhere)
    iCity = pCity.getID()
    lRoutes = CityUtils.getCityDictionaryValue(pCity, "SupplyRoutes")
    if(lRoutes != -1):
        for tRoute in lRoutes:
            if(tRoute[0] == iCity):
                # This is the source city
                iDeltaFood = iDeltaFood - tRoute[1]
            elif(tRoute[3] == iCity):
                # This is the destination city
                iDeltaFood = iDeltaFood + tRoute[1]
    
    # Food Production Modification
    iFoodDifferenceOrig = pCity.foodDifference(True)
    iFoodDifference = iFoodDifferenceOrig + iDeltaFood
    
    if(iDeltaFood != 0):
        if((iFoodDifference < 0) and (iFoodDifferenceOrig < 0)):                            
            # All comes from food.
            return iDeltaFood
        elif((iFoodDifference > 0) and (iFoodDifferenceOrig < 0)):
            if(pCity.isFoodProduction()):
                return (iFoodDifferenceOrig * -1)
            else:
                return iDeltaFood
        elif((iFoodDifference < 0) and (iFoodDifferenceOrig > 0)):
            return iDeltaFood
        elif((iFoodDifference > 0) and (iFoodDifferenceOrig > 0)):
            if(pCity.isFoodProduction()):
                return 0
            else:
                return iDeltaFood


# Resource-linked Buildings
# Supply Routes
# Extra unhealthiness
def getExtraProductionForCity(pCity):
    iDeltaProduction = 0
    # Resource-linked Buildings
    for iBuilding in range(gc.getNumBuildingInfos()):
        if(pCity.hasBuilding(iBuilding)):
            eBuilding = gc.getBuildingInfo(iBuilding).getType()
            if(dResourceLinkedBuildings.has_key(eBuilding)):
                dDependencies = dResourceLinkedBuildings[eBuilding]
                # ResourceBaseBuildingYieldsDict["BUILDING_HELIUM3_EXCHANGE"] = { "BONUS_HELIUM3": ((0, 0, (0, 0, 2)), True) }
                for sResource in dDependencies.keys():
                    tYieldBonus = dDependencies[sResource]
                    # Test for Food
                    if(tYieldBonus[0][1] == 0):
                        continue
                    iBonus = gc.getInfoTypeForString(sResource)
                    bPerResource = tYieldBonus[1]
                    # Get resource count
                    iOrigX = pCity.plot().getX()
                    iOrigY = pCity.plot().getY()
                    iWorkRadius = 2
                    iCount = 0
                    # Written this way in case I want to change the algorithm later
                    for iX in range (-2, 3):
                        for iY in range (-2, 3):
                            if(abs(iY) + abs(iX) <= iWorkRadius + 1):
                                x = iOrigX + iX
                                y = iOrigY + iY
                                pPlot = gc.getMap().plot(x,y)
                                if(pPlot.getBonusType(pPlayer.getTeam()) == iBonus):
                                    # This gaurantees that only one city gets credit for each He3, even with overlapping zones
                                    if(pPlot.getWorkingCity().getID() == pCity.getID()):
                                        # Verify that the bonus is on the player's network
                                        if(pPlot.isBonusNetwork(pPlayer.getTeam())):
                                            iImp = pPlot.getImprovementType()
                                            if(iImp != -1):
                                                if(gc.getImprovementInfo(iImp).isImprovementBonusMakesValid(iBonus)):
                                                    iCount = iCount + 1
                    # Add bonus
                    if(iCount > 0):
                        iProduction = tYieldBonus[0][1]
                        if(bPerResource):
                            iProduction = iProduction * iCount
                        if(iFood != 0):
                            iDeltaProduction = iDeltaProduction + iProduction
                        if(iFood > 0):
                            iDeltaProduction = iDeltaProduction + ((iProduction * pCity.getYieldRateModifier(0)) / 100)
                            
    # Supply Routes (Validated elsewhere)
    iCity = pCity.getID()
    lRoutes = CityUtils.getCityDictionaryValue(pCity, "SupplyRoutes")
    if(lRoutes != -1):
        for tRoute in lRoutes:
            if(tRoute[0] == iCity):
                # This is the source city
                iDeltaFood = iDeltaFood - tRoute[2]
            elif(tRoute[3] == iCity):
                # This is the destination city
                iDeltaFood = iDeltaFood + tRoute[2]
    
    if(pCity.isFoodProduction()):
        iFoodDifferenceOrig = pCity.foodDifference(True)
        iFoodDifference = iFoodDifferenceOrig + iFoodDelta
        
        if(iFoodDelta != 0):
            if((iFoodDifference < 0) and (iFoodDifferenceOrig < 0)):                            
                # All comes from food.
                pCity.changeFood(iFoodDelta)
            elif((iFoodDifference > 0) and (iFoodDifferenceOrig <0)):
                if(pCity.isFoodProduction()):
                    pCity.changeFood(iFoodDifferenceOrig * -1)
                    pCity.changeProduction(iFoodDifference)
                else:
                    pCity.changeFood(iFoodDelta)
            elif((iFoodDifference < 0) and (iFoodDifferenceOrig > 0)):
                if(pCity.isFoodProduction()):
                    pCity.changeProduction(-1 * iFoodDifferenceOrig)
                    pCity.changeFood(iFoodDifference)
                else:
                    pCity.changeFood(iFoodDelta)
            elif((iFoodDifference > 0) and (iFoodDifferenceOrig > 0)):
                if(pCity.isFoodProduction()):
                    pCity.changeProduction(iFoodDelta)
                else:
                    pCity.changeFood(iFoodDelta)
                
                            
# Resource-linked Buildings    
def getExtraGoldForCity(pCity):
    
# Resource-linked Buildings
def getExtraCultureForCity(pCity):

# Resource-linked Buildings
def getExtraResearchForCity(pCity):


# All cities' extra research
def getExtraResearchForPlayer(pPlayer):

# All cities' extra gold
def getExtraGoldForPlayer(pPlayer):
    

def getRealProductionForCity(pCity):
    
def getRealResearch(pPlayer):


def getTotalExtraFoodForCity(pCity:
        iDeltaFood = 0
    # Resource-linked Buildings
    for iBuilding in range(gc.getNumBuildingInfos()):
        if(pCity.hasBuilding(iBuilding)):
            eBuilding = gc.getBuildingInfo(iBuilding).getType()
            if(dResourceLinkedBuildings.has_key(eBuilding)):
                dDependencies = dResourceLinkedBuildings[eBuilding]
                # ResourceBaseBuildingYieldsDict["BUILDING_HELIUM3_EXCHANGE"] = { "BONUS_HELIUM3": ((0, 0, (0, 0, 2)), True) }
                for sResource in dDependencies.keys():
                    tYieldBonus = dDependencies[sResource]
                    # Test for Food
                    if(tYieldBonus[0][0] == 0):
                        continue
                    iBonus = gc.getInfoTypeForString(sResource)
                    bPerResource = tYieldBonus[1]
                    # Get resource count
                    iOrigX = pCity.plot().getX()
                    iOrigY = pCity.plot().getY()
                    iWorkRadius = 2
                    iCount = 0
                    # Written this way in case I want to change the algorithm later
                    for iX in range (-2, 3):
                        for iY in range (-2, 3):
                            if(abs(iY) + abs(iX) <= iWorkRadius + 1):
                                x = iOrigX + iX
                                y = iOrigY + iY
                                pPlot = gc.getMap().plot(x,y)
                                if(pPlot.getBonusType(pPlayer.getTeam()) == iBonus):
                                    # This gaurantees that only one city gets credit for each He3, even with overlapping zones
                                    if(pPlot.getWorkingCity().getID() == pCity.getID()):
                                        # Verify that the bonus is on the player's network
                                        if(pPlot.isBonusNetwork(pPlayer.getTeam())):
                                            iImp = pPlot.getImprovementType()
                                            if(iImp != -1):
                                                if(gc.getImprovementInfo(iImp).isImprovementBonusMakesValid(iBonus)):
                                                    iCount = iCount + 1
                    # Add bonus
                    if(iCount > 0):
                        iFood = tYieldBonus[0][0]
                        if(bPerResource):
                            iFood = iFood * iCount
                        if(iFood != 0):
                            iDeltaFood = iDeltaFood + iFood
                        if(iFood > 0):
                            iDeltaFood = iDeltaFood + ((iFood * pCity.getYieldRateModifier(0)) / 100)
    
    # Extra Unhealthiness
    iExtraUnhealthiness = CityUtils.getCityDictionaryValue(pCity, "extraUnhealthiness")
    if(iExtraUnhealthiness != 0):
        if(iExtraUnhealthiness < 0):
            iExtraHealthiness = -1 * iExtraUnhealthiness
            iExtraUnhealthiness = 0
        else:
            iExtraHealthiness = 0
        iGoodHealth = pCity.goodHealth()
        iBadHealth = pCity.badHealth(False)
        iAdjBadHealth = iBadHealth + iExtraUnhealthiness
        iAdjGoodHealth = iGoodHealth + iExtraHealthiness
        iFoodCalc = min((iGoodHealth - iBadHealth, 0))
        iFoodReal = min(((iGoodHealth + iExtraHealthiness) - (iBadHealth + iExtraUnhealthiness)), 0)
        iDeltaFood = iFoodReal - iFoodCalc
        
        
    # Supply Routes (Validated elsewhere)
    iCity = pCity.getID()
    lRoutes = CityUtils.getCityDictionaryValue(pCity, "SupplyRoutes")
    if(lRoutes != -1):
        for tRoute in lRoutes:
            if(tRoute[0] == iCity):
                # This is the source city
                iDeltaFood = iDeltaFood - tRoute[1]
            elif(tRoute[3] == iCity):
                # This is the destination city
                iDeltaFood = iDeltaFood + tRoute[1]
                
    return iDeltaFood






# Extra Unhealthiness support
        for iDX in range(pPlayer.getNumCities()):
            pCity = pPlayer.getCity(iDX)
            iExtraUnhealthiness = CityUtils.getCityDictionaryValue(pCity, "extraUnhealthiness")
            if(iExtraUnhealthiness != 0):
                if(iExtraUnhealthiness < 0):
                    iExtraHealthiness = -1 * iExtraUnhealthiness
                    iExtraUnhealthiness = 0
                else:
                    iExtraHealthiness = 0
                iGoodHealth = pCity.goodHealth()
                iBadHealth = pCity.badHealth(False)
                iAdjBadHealth = iBadHealth + iExtraUnhealthiness
                iAdjGoodHealth = iGoodHealth + iExtraHealthiness
                iFoodCalc = min((iGoodHealth - iBadHealth, 0))
                iFoodReal = min(((iGoodHealth + iExtraHealthiness) - (iBadHealth + iExtraUnhealthiness)), 0)
                iFoodDelta = iFoodReal - iFoodCalc
                
                iFoodDifferenceOrig = pCity.foodDifference(True)
                iFoodDifference = iFoodDifferenceOrig + iFoodDelta
                
                if(iFoodDelta != 0):
                    if((iFoodDifference < 0) and (iFoodDifferenceOrig < 0)):                            
                        # All comes from food.
                        pCity.changeFood(iFoodDelta)
                    elif((iFoodDifference > 0) and (iFoodDifferenceOrig <0)):
                        if(pCity.isFoodProduction()):
                            pCity.changeFood(iFoodDifferenceOrig * -1)
                            pCity.changeProduction(iFoodDifference)
                        else:
                            pCity.changeFood(iFoodDelta)
                    elif((iFoodDifference < 0) and (iFoodDifferenceOrig > 0)):
                        if(pCity.isFoodProduction()):
                            pCity.changeProduction(-1 * iFoodDifferenceOrig)
                            pCity.changeFood(iFoodDifference)
                        else:
                            pCity.changeFood(iFoodDelta)
                    elif((iFoodDifference > 0) and (iFoodDifferenceOrig > 0)):
                        if(pCity.isFoodProduction()):
                            pCity.changeProduction(iFoodDelta)
                        else:
                            pCity.changeFood(iFoodDelta)
        
        
        # Resource-Linked Buildings
        dResourceLinkedBuildings = Config.getResourceBaseBuildingYieldsDict()
        for iDX in range(pPlayer.getNumCities()):
            #message = "Checking buildings in city %s" %(pCity.getName())
            #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
                                    
            pCity = pPlayer.getCity(iDX)
            for iBuilding in range(gc.getNumBuildingInfos()):
                if(pCity.hasBuilding(iBuilding)):
                    eBuilding = gc.getBuildingInfo(iBuilding).getType()
                    
                    #message = "Found building %s" %(eBuilding)
                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)

                    if(dResourceLinkedBuildings.has_key(eBuilding)):
                        
                        #message = "Found resource dependent building %s" %(eBuilding)
                        #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
            
                        dDependencies = dResourceLinkedBuildings[eBuilding]
                        # ResourceBaseBuildingYieldsDict["BUILDING_HELIUM3_EXCHANGE"] = { "BONUS_HELIUM3": ((0, 0, (0, 0, 2)), True) }
                        for sResource in dDependencies.keys():
                            tYieldBonus = dDependencies[sResource]
                            iBonus = gc.getInfoTypeForString(sResource)
                            bPerResource = tYieldBonus[1]
                            # Get resource count
                            iOrigX = pCity.plot().getX()
                            iOrigY = pCity.plot().getY()
                            iWorkRadius = 2
                            iCount = 0
                            # Written this way in case I want to change the algorithm later
                            for iX in range (-2, 3):
                                for iY in range (-2, 3):
                                    if(abs(iY) + abs(iX) <= iWorkRadius + 1):
                                        x = iOrigX + iX
                                        y = iOrigY + iY
                                        pPlot = gc.getMap().plot(x,y)
                                        if(pPlot.getBonusType(pPlayer.getTeam()) == iBonus):
                                            
                                            #bRes = pPlot.isPlotGroupConnectedBonus(pPlayer.getID(), iBonus)
                                            #bBNet = pPlot.isBonusNetwork(pPlayer.getTeam())
                                            #message = "Resource at %d, %d returns %s and %s" %(x, y, str(bRes), str(bBNet))
                                            #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),pCity.getX(),pCity.getY(),True,True)
            
                                            # This gaurantees that only one city gets credit for each He3, even with overlapping zones
                                            if(pPlot.getWorkingCity().getID() == pCity.getID()):
                                                #if(pPlot.isBeingWorked()):
                                                #if(pPlot.isPlotGroupConnectedBonus(pPlayer.getID(), iBonus)):  # This will return true if any bonus of that type is connected, and that stile has a route to the network
                                                
                                                # Verify that the bonus is on the player's network
                                                if(pPlot.isBonusNetwork(pPlayer.getTeam())):
                                                    iImp = pPlot.getImprovementType()
                                                    if(iImp != -1):
                                                        if(gc.getImprovementInfo(iImp).isImprovementBonusMakesValid(iBonus)):
                                                            iCount = iCount + 1
                            
                        
                            # Add bonus
                            if(iCount > 0):
                                iFood = tYieldBonus[0][0]
                                iProd = tYieldBonus[0][1]
                                tCommerce = tYieldBonus[0][2]
                                iGold = tCommerce[0]
                                iRes = tCommerce[1]
                                iCult = tCommerce[2]
                                if(bPerResource):
                                    iFood = iFood * iCount
                                    iProd = iProd * iCount
                                    iGold = iGold * iCount
                                    iRes = iRes * iCount
                                    iCult = iCult * iCount
                                
                                if(iFood != 0):
                                    iFood = iFood + ((iFood * pCity.getYieldRateModifier(0)) / 100)
                                    #message = "Adding %d food to city %s" %(iFood, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    pCity.changeFood(iFood)
                                if(iProd != 0):
                                    iProd = iProd + ((iProd * pCity.getYieldRateModifier(1)) / 100)
                                    #message = "Adding %d production to city %s" %(iProd, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    pCity.changeProduction(iProd)
                                if(iGold != 0):
                                    iGold = iGold + ((iGold * pCity.getCommerceRateModifier(0)) / 100)
                                    #message = "Adding %d gold 'to' city %s" %(iGold, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    pPlayer.changeGold(iGold)
                                if(iRes != 0):
                                    iRes = iRes + ((iRes * pCity.getCommerceRateModifier(1)) / 100)
                                    #message = "Adding %d research 'to' city %s" %(iRes, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    gc.getTeam(pPlayer.getTeam()).changeResearchProgress(pPlayer.getCurrentResearch(), iRes, pPlayer.getID())
                                if(iCult != 0):
                                    iCult = iCult + ((iCult * pCity.getCommerceRateModifier(2)) / 100)
                                    #message = "Adding %d culture to city %s" %(iCult, pCity.getName())
                                    #CyInterface().addMessage(pCity.getOwner(),True,len(message),message,'AS2D_BOMBARDED',0,'Art/Interface/Buttons/Units/Fighter.dds',ColorTypes(7),iOrigX,iOrigY,True,True)
                                    pCity.changeCulture(pPlayer.getID(), iCult, True)
