## Colonize the Moon Mod Project
## Changes copyright Shin 2005
## (if any portion of this copyright should be found to be void, the remainder shall remain in full effect)
##
## PlayerUtils
#
# --Shin
# (aka. Belizan)

from CvPythonExtensions import *
from types import *
import random
import cPickle
import TechUtils
import UnitUtils
import CityUtils
import DebugUtils

gc = CyGlobalContext()



def initPlayerDictionary(pPlayer):
    playerDict = {}
    pPlayer.setScriptData(cPickle.dumps(playerDict))
    
def getPlayerDictionary(pPlayer):
    if(pPlayer.getScriptData() != ""):
        playerDict = cPickle.loads(pPlayer.getScriptData())
    else:
        playerDict = {}
    return playerDict
    
def setPlayerDictionary(pPlayer, playerDict):
    pPlayer.setScriptData(cPickle.dumps(playerDict))
    
    
def getPlayerDictionaryValue(pPlayer, sValue):
    playerDict = getPlayerDictionary(pPlayer)
    if(playerDict.has_key(sValue)):
        return playerDict[sValue]
    else:
        return -1
        
def setPlayerDictionaryValue(pPlayer, sValKey, oValue):
    playerDict = getPlayerDictionary(pPlayer)
    playerDict[sValKey] = oValue
    setPlayerDictionary(pPlayer, playerDict)
    
def increasePlayerDictionaryValue(pPlayer, sValKey, iIncrement):
    playerDict = getPlayerDictionary(pPlayer)
    if(playerDict.has_key(sValKey)):
        playerDict[sValKey] = playerDict[sValKey] + iIncrement
    else:
        playerDict[sValKey] = iIncrement
    setPlayerDictionary(pPlayer, playerDict)
    return playerDict[sValKey]
    
def removePlayerDictionaryValue(pPlayer, sValKey):
    playerDict = getPlayerDictionary(pPlayer)
    if(playerDict.has_key(sValKey)):
        del playerDict[sValKey]
    setPlayerDictionary(pPlayer, playerDict)
    
    
def addToCounter(pPlayer, sCounter, iVal):
    playerDict = getPlayerDictionary(pPlayer)
    if(playerDict.has_key(sCounter)):
        playerDict[sCounter] = playerDict[sCounter] + iVal
    else:
        playerDict[sCounter] = iVal
    setPlayerDictionary(pPlayer, playerDict)
    return playerDict[sCounter]
    
def resetCounter(pPlayer, sCounter):
    playerDict = getPlayerDictionary(pPlayer)
    playerDict[sCounter] = 0
    setPlayerDictionary(pPlayer, playerDict)
    
def getCounter(pPlayer, sCounter):
    playerDict = getPlayerDictionary(pPlayer)
    if(playerDict.has_key(sCounter)):
        return playerDict[sCounter]
    else:
        return 0


def getGameOptionsDictionary(pPlayer):
    playerDict = getPlayerDictionary(pPlayer)
    if(playerDict.has_key("GameOptions")):
        return playerDict["GameOptions"]
    else:
        return {}
    
def setGameOptionsDictionary(pPlayer, dOptions):
    playerDict = getPlayerDictionary(pPlayer)
    playerDict["GameOptions"] = dOptions
    setPlayerDictionary(pPlayer, playerDict)
    
def getGameOption(pPlayer, sValKey):
    dOptions = getGameOptionsDictionary(pPlayer)
    if(dOptions.has_key(sValKey)):
        return dOptions[sValKey]
    else:
        return -1
    
def setGameOption(pPlayer, sValKey, sValue):
    dOptions = getGameOptionsDictionary(pPlayer)
    dOptions[sValKey] = sValue
    setGameOptionsDictionary(pPlayer, dOptions)
    
def addSignToList(pPlayer, pPlot):
    lSigns = PlayerUtils.getPlayerDictionaryValue(pPlayer, "SignsList")
    if(lSigns != -1):
        lSigns.append((pPlot.getX(), pPlot.getY()))
    else:
        lSigns = []
        lSigns.append((pPlot.getX(), pPlot.getY()))
    PlayerUtils.setPlayerDictionaryValue(pPlayer, "SignsList", lSigns)

# so inefficient it's funny
def removeSignFromList(pPlayer, pPlot):
    lSigns = PlayerUtils.getPlayerDictionaryValue(pPlayer, "SignsList")
    if(lSigns == -1):
        return
    else:
        for tPair in lSigns:
            if(pPlot.at(tPair[0], tPair[1])):
                lSigns.remove(tPair)
                break
        PlayerUtils.setPlayerDictionaryValue(pPlayer, "SignsList", lSigns)

def removeSiteFromList(pPlayer, pPlot):
    lSites = getPlayerDictionaryValue(pPlayer, "SiteList")
    if(lSites != -1):
        for tPair in lSites:
            if(pPlot.at(tPair[0], tPair[1])):
                lSites.remove(tPair)
                break
        setPlayerDictionaryValue(pPlayer, "SiteList", lSites)
    
def addSiteToList(pPlayer, pPlot):
    lSites = getPlayerDictionaryValue(pPlayer, "SiteList")
    if(lSites == -1):
        lSites = []
    lSites.append((pPlot.getX(), pPlot.getY()))
    setPlayerDictionaryValue(pPlayer, "SiteList", lSites)
    


# Returns True/False if player is allowed to research this tech (ever)
def hasAccessToTech(pPlayer, iTech):
    ePlayerType = [ "NONE" ]
    
    if(hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
        ePlayerType = [ "MILITARY", "HAPPINESS" ]
    if(hasTrait(pPlayer, "TRAIT_ECONOMIC")):
        ePlayerType = [ "COMMERCIAL" ]
    if(hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
        ePlayerType = [ "SCIENTIFIC" ]
        
    eTech = TechUtils.getTagEncodedType(iTech)
    
    if(eTech in ePlayerType):
        return True
    else:
        if(eTech == "NONE"):
            return not TechUtils.isSuchTaggedTech(iTech, ePlayerType)
        else:
            return False
    
    
# Returns True/False if player is allowed to train this unit (ever)
def hasAccessToUnit(pPlayer, iUnit):
    ePlayerType = [ "NONE" ]
    eExclPlayerType = []
    
    if(hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
        ePlayerType = [ "MILITARY" ]
        eExclPlayerType = [ "HAPPINESS" ]
    if(hasTrait(pPlayer, "TRAIT_ECONOMIC")):
        ePlayerType = [ "COMMERCIAL" ]
    if(hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
        ePlayerType = [ "SCIENTIFIC" ]
        
    eUnit = UnitUtils.getTagEncodedType(iUnit)
    
    if(eUnit in ePlayerType):
        return True
    elif(eUnit in eExclPlayerType):
        return False
    else:
        if(eUnit == "NONE"):
            return not UnitUtils.isSuchTaggedUnit(iUnit, ePlayerType)
        else:
            return False
    
    
# Returns True/False if player is allowed to construct this building (ever)
def hasAccessToBuilding(pPlayer, iBuilding):
    ePlayerType = [ "NONE" ]
    eExclPlayerType = []
    
    if(hasTrait(pPlayer, "TRAIT_INDEPENDENT")):
        ePlayerType = [ "MILITARY" ]
        eExclPlayerType = [ "HAPPINESS" ]
    if(hasTrait(pPlayer, "TRAIT_ECONOMIC")):
        ePlayerType = [ "COMMERCIAL" ]
    if(hasTrait(pPlayer, "TRAIT_SCIENTIFIC")):
        ePlayerType = [ "SCIENTIFIC" ]
        
    eBuilding = CityUtils.getTagEncodedType(iBuilding)
    DebugUtils.debugStr("eBuilding: %s" %(eBuilding))
    if(eBuilding in ePlayerType):
        DebugUtils.debugStr("%s Allowed for %s" %(eBuilding, pPlayer.getName()))
        return True
    elif(eBuilding in eExclPlayerType):
        DebugUtils.debugStr("%s denied for %s" %(eBuilding, pPlayer.getName()))
        return False
    else:
        if(eBuilding == "NONE"):
            lTags = CityUtils.getAllTaggedBuildings(iBuilding)
            DebugUtils.debugStr("None Base for %s, other tags: %s" %(eBuilding, lTags))
            for eTag in ePlayerType:
                if eTag in lTags:
                    return False
            if "HAPPINESS" in lTags:
                return False
            return True
        elif(eBuilding == "HAPPINESS"):
            lTags = CityUtils.getAllTaggedBuildings(iBuilding)
            DebugUtils.debugStr("Happiness Base for %s, other tags: %s" %(eBuilding, lTags))
            for eTag in ePlayerType:
                if eTag in lTags:
                    return False
            return True
        else:
            return False
    
# Returns True/False if player is allowed to construct this project (ever)
def hasAccessToProject(pPlayer, iProject):
    return True


def colorizeCityPlots(pPlayer):
    CyEngine().clearColoredPlots(PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER)
    for iDX in range(pPlayer.getNumCities()):
        pCity = pPlayer.getCity(iDX)
        iX = pCity.getX()
        iY = pCity.getY()
        sColor = "COLOR_WHITE"
        if(CyInterface().getHeadSelectedCity() != None):
            if(pCity.getID() == CyInterface().getHeadSelectedCity().getID()):
                sColor = "COLOR_YELLOW"
        for x in range(-2, 3):
            for y in range(-2, 3):
                pPlot = gc.getMap().plot(iX + x, iY + y)
                if(not pPlot.isNone()):
                    if(pPlot.getWorkingCity().getID() == pCity.getID()):
                        CyEngine().addColoredPlotAlt(pPlot.getX(), pPlot.getY(), PlotStyles.PLOT_STYLE_BOX_OUTLINE, PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER, sColor, .7)

def colorizeCitySites(pPlayer):
    #CyEngine().clearColoredPlots(PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER)
    lSites = getPlayerDictionaryValue(pPlayer, "SiteList")
    if(lSites != -1):
        for tPair in lSites:
            iX = tPair[0]
            iY = tPair[1]
            sColor = "COLOR_GREEN"
            for x in range(-2, 3):
                for y in range(-2, 3):
                    if(abs(x) + abs(y) != 4):
                        pPlot = gc.getMap().plot(iX + x, iY + y)
                        if(not pPlot.isNone()):
                            CyEngine().addColoredPlotAlt(pPlot.getX(), pPlot.getY(), PlotStyles.PLOT_STYLE_BOX_OUTLINE, PlotLandscapeLayers.PLOT_LANDSCAPE_LAYER_WORLD_BUILDER, sColor, .7)
                        


def getLeaderInfo(pPlayer):
    iLd = pPlayer.getLeaderType()
    pLeaderHeadInfo = gc.getLeaderHeadInfo(iLd)
    return pLeaderHeadInfo
    
def getFlavorValue(pPlayer, eFlavor):
    pLeader = getLeaderInfo(pPlayer)
    
    # gc.getFlavorTypes(int)  gc.getNumFlavorTypes()
    for iDX in range(gc.getNumFlavorTypes()):
        if(gc.getFlavorTypes(iDX) == eFlavor):
            iFlavor = iDX
            break
            
    return pLeader.getFlavorValue(iFlavor)
    
def hasTrait(pPlayer, eTrait):
    pLeader = getLeaderInfo(pPlayer)
    iTrait = gc.getInfoTypeForString(eTrait)
    if(iTrait == -1):
        return False
        
    return pLeader.hasTrait(iTrait)
    
    