from CvPythonExtensions import *
from types import *
import UnitUtils
import Config

# globals
gc = CyGlobalContext()

# Turrets--One per city
# There are two aspects to implementing turrets.
# First, that only one can be build per city.  This is accomplished by overriding cannotTrain in CvGameUtils.
# Second, that they be automatically upgraded as new techs are discovered*
# *I've changed this to an end of turn check, because cities can be captured, etc.(though I could test in both places, really)
# --Shin
# (aka. Belizan)


def isUpgradeTest(pUnit):
    unitDict = UnitUtils.getUnitDictionary(pUnit)
    if(unitDict.has_key("upgradeTest")):
        return unitDict["upgradeTest"]
    else:
        return False

def setUpgradeTest(pUnit):
    unitDict = UnitUtils.getUnitDictionary(pUnit)
    unitDict["upgradeTest"] = True
    UnitUtils.setUnitDictionary(pUnit, unitDict)
    
def clearUpgradeTest(pUnit):
    unitDict = UnitUtils.getUnitDictionary(pUnit)
    unitDict["upgradeTest"] = False
    UnitUtils.setUnitDictionary(pUnit, unitDict)
    
    
def isTurret(pUnit):
    if(isinstance(pUnit, NoneType)):
        return False
        
    if(gc.getUnitClassInfo(pUnit.getUnitClassType()).getType() in Config.TurretUnitClasses):
        return True
    return False
    

# Restrict building of turrets to one per city
def cannotTrain(iUnit, pCity):
    eUnit = gc.getUnitInfo(iUnit).getType()
    eUnitClass = gc.getUnitClassInfo(gc.getUnitInfo(iUnit).getUnitClassType()).getType()
    #return False
    if(eUnitClass in Config.TurretUnitClasses):
        if(isUpgradeTest(pCity)):
            return False
        for iUnitNum in range(pCity.plot().getNumUnits()):
            pUnit = pCity.plot().getUnit(iUnitNum)
            if(isTurret(pUnit)):
                return True
    return False


# Upgrade a turret if possible
def upgradeTurret(pUnit):
    if(isTurret(pUnit)):
        iX = pUnit.getX()
        iY = pUnit.getY()
        iCount = 0
        # Unit must have at least 1 movement point to upgrade
        pUnit.setMoves(1)
        pCity = pUnit.plot()
        if(pCity.isCity()):
            #Assert: Must be true
            pCity = pCity.getPlotCity()
        else:
            pUnit.finishMoves()
            return None 
            
        # Unit must be trainable to be upgradable to (CannotTrain must return false)
        # Note that executions are in different modules/apartments, so you can't use a global for this purpose
        setUpgradeTest(pCity)
        
        iUnitT = pUnit.getUnitType()
        eUnit = gc.getUnitInfo(iUnitT).getType()
            
        for iUnit in range(gc.getNumUnitInfos()):
            eUnitClass = gc.getUnitClassInfo(gc.getUnitInfo(iUnit).getUnitClassType()).getType()
            iUnitClass = gc.getUnitInfo(iUnit).getUnitClassType()
            pUnitInfo = gc.getUnitInfo(pUnit.getUnitType()) 
            
            if(pUnitInfo.getUpgradeUnitClass(iUnitClass)): 
                
                setUpgradeTest(pCity)
                upgradeTypes = pCity.allUpgradesAvailable(pUnit.getUnitType(), 5)
                
                if((upgradeTypes != -1) and (upgradeTypes != iUnitT)):
                    UnitUtils.upgradeUnit(pUnit, upgradeTypes)
                    pUnit.finishMoves()
                    clearUpgradeTest(pCity)
                    return None
                    
        clearUpgradeTest(pCity)
        pUnit.finishMoves()
        return None 